@@ -62,6 +62,7 @@ var doSimulcast = (getQueryStringValue("simulcast") === "yes" || getQueryStringV
 var doSimulcast2 = (getQueryStringValue("simulcast2") === "yes" || getQueryStringValue("simulcast2") === "true");
 var acodec = (getQueryStringValue("acodec") !== "" ? getQueryStringValue("acodec") : null);
 var vcodec = (getQueryStringValue("vcodec") !== "" ? getQueryStringValue("vcodec") : null);
+var vprofile = (getQueryStringValue("vprofile") !== "" ? getQueryStringValue("vprofile") : null);
 var simulcastStarted = false;
 
 $(document).ready(function() {
@@ -99,15 +100,19 @@ $(document).ready(function() {
 									echotest = pluginHandle;
 									Janus.log("Plugin attached! (" + echotest.getPlugin() + ", id=" + echotest.getId() + ")");
 									// Negotiate WebRTC
-									var body = { "audio": true, "video": true };
+									var body = { audio: true, video: true };
 									// We can try and force a specific codec, by telling the plugin what we'd prefer
 									// For simplicity, you can set it via a query string (e.g., ?vcodec=vp9)
 									if(acodec)
 										body["audiocodec"] = acodec;
 									if(vcodec)
 										body["videocodec"] = vcodec;
-									Janus.debug("Sending message (" + JSON.stringify(body) + ")");
-									echotest.send({"message": body});
+									// For the codecs that support them (VP9 and H.264) you can specify a codec
+									// profile as well (e.g., ?vprofile=2 for VP9, or ?vprofile=42e01f for H.264)
+									if(vprofile)
+										body["videoprofile"] = vprofile;
+									Janus.debug("Sending message:", body);
+									echotest.send({ message: body });
 									Janus.debug("Trying a createOffer too (audio/video sendrecv)");
 									echotest.createOffer(
 										{
@@ -119,13 +124,12 @@ $(document).ready(function() {
 											simulcast: doSimulcast,
 											simulcast2: doSimulcast2,
 											success: function(jsep) {
-												Janus.debug("Got SDP!");
-												Janus.debug(jsep);
-												echotest.send({"message": body, "jsep": jsep});
+												Janus.debug("Got SDP!", jsep);
+												echotest.send({ message: body, jsep: jsep });
 											},
 											error: function(error) {
 												Janus.error("WebRTC error:", error);
-												bootbox.alert("WebRTC error... " + JSON.stringify(error));
+												bootbox.alert("WebRTC error... " + error.message);
 											}
 										});
 									$('#start').removeAttr('disabled').html("Stop")
@@ -175,19 +179,17 @@ $(document).ready(function() {
 										" packets on this PeerConnection (" + lost + " lost packets)");
 								},
 								onmessage: function(msg, jsep) {
-									Janus.debug(" ::: Got a message :::");
-									Janus.debug(msg);
-									if(jsep !== undefined && jsep !== null) {
-										Janus.debug("Handling SDP as well...");
-										Janus.debug(jsep);
-										echotest.handleRemoteJsep({jsep: jsep});
+									Janus.debug(" ::: Got a message :::", msg);
+									if(jsep) {
+										Janus.debug("Handling SDP as well...", jsep);
+										echotest.handleRemoteJsep({ jsep: jsep });
 									}
 									var result = msg["result"];
-									if(result !== null && result !== undefined) {
+									if(result) {
 										if(result === "done") {
 											// The plugin closed the echo test
 											bootbox.alert("The Echo Test is over");
-											if(spinner !== null && spinner !== undefined)
+											if(spinner)
 												spinner.stop();
 											spinner = null;
 											$('#myvideo').remove();
@@ -203,8 +205,6 @@ $(document).ready(function() {
 										// Any loss?
 										var status = result["status"];
 										if(status === "slow_link") {
-											//~ var bitrate = result["bitrate"];
-											//~ toastr.warning("The bitrate has been cut to " + (bitrate/1000) + "kbps", "Packet loss?", {timeOut: 2000});
 											toastr.warning("Janus apparently missed many packets we sent, maybe we should reduce the bitrate", "Packet loss?", {timeOut: 2000});
 										}
 									}
@@ -221,8 +221,7 @@ $(document).ready(function() {
 									}
 								},
 								onlocalstream: function(stream) {
-									Janus.debug(" ::: Got a local stream :::");
-									Janus.debug(stream);
+									Janus.debug(" ::: Got a local stream :::", stream);
 									if($('#myvideo').length === 0) {
 										$('#videos').removeClass('hide').show();
 										$('#videoleft').append('<video class="rounded centered" id="myvideo" width=320 height=240 autoplay playsinline muted="muted"/>');
@@ -249,7 +248,7 @@ $(document).ready(function() {
 										}
 									}
 									var videoTracks = stream.getVideoTracks();
-									if(videoTracks === null || videoTracks === undefined || videoTracks.length === 0) {
+									if(!videoTracks || videoTracks.length === 0) {
 										// No webcam
 										$('#myvideo').hide();
 										if($('#videoleft .no-video-container').length === 0) {
@@ -265,8 +264,7 @@ $(document).ready(function() {
 									}
 								},
 								onremotestream: function(stream) {
-									Janus.debug(" ::: Got a remote stream :::");
-									Janus.debug(stream);
+									Janus.debug(" ::: Got a remote stream :::", stream);
 									var addButtons = false;
 									if($('#peervideo').length === 0) {
 										addButtons = true;
@@ -277,7 +275,7 @@ $(document).ready(function() {
 											$('#waitingvideo').remove();
 											if(this.videoWidth)
 												$('#peervideo').removeClass('hide').show();
-											if(spinner !== null && spinner !== undefined)
+											if(spinner)
 												spinner.stop();
 											spinner = null;
 											var width = this.videoWidth;
@@ -287,7 +285,7 @@ $(document).ready(function() {
 									}
 									Janus.attachMediaStream($('#peervideo').get(0), stream);
 									var videoTracks = stream.getVideoTracks();
-									if(videoTracks === null || videoTracks === undefined || videoTracks.length === 0) {
+									if(!videoTracks || videoTracks.length === 0) {
 										// No remote video
 										$('#peervideo').hide();
 										if($('#videoright .no-video-container').length === 0) {
@@ -313,7 +311,7 @@ $(document).ready(function() {
 												$('#toggleaudio').html("Disable audio").removeClass("btn-success").addClass("btn-danger");
 											else
 												$('#toggleaudio').html("Enable audio").removeClass("btn-danger").addClass("btn-success");
-											echotest.send({"message": { "audio": audioenabled }});
+											echotest.send({ message: { audio: audioenabled }});
 										});
 									$('#togglevideo').click(
 										function() {
@@ -322,7 +320,7 @@ $(document).ready(function() {
 												$('#togglevideo').html("Disable video").removeClass("btn-success").addClass("btn-danger");
 											else
 												$('#togglevideo').html("Enable video").removeClass("btn-danger").addClass("btn-success");
-											echotest.send({"message": { "video": videoenabled }});
+											echotest.send({ message: { video: videoenabled }});
 										});
 									$('#toggleaudio').parent().removeClass('hide').show();
 									$('#bitrate a').click(function() {
@@ -334,7 +332,7 @@ $(document).ready(function() {
 											Janus.log("Capping bandwidth to " + bitrate + " via REMB");
 										}
 										$('#bitrateset').html($(this).html() + '<span class="caret"></span>').parent().removeClass('open');
-										echotest.send({"message": { "bitrate": bitrate }});
+										echotest.send({ message: { bitrate: bitrate }});
 										return false;
 									});
 									if(Janus.webRTCAdapter.browserDetails.browser === "chrome" || Janus.webRTCAdapter.browserDetails.browser === "firefox" ||
@@ -343,7 +341,6 @@ $(document).ready(function() {
 										bitrateTimer = setInterval(function() {
 											// Display updated bitrate, if supported
 											var bitrate = echotest.getBitrate();
-											//~ Janus.debug("Current bitrate is " + echotest.getBitrate());
 											$('#curbitrate').text(bitrate);
 											// Check if the resolution changed too
 											var width = $("#peervideo").get(0).videoWidth;
@@ -359,12 +356,12 @@ $(document).ready(function() {
 									$('#datasend').removeAttr('disabled');
 								},
 								ondata: function(data) {
-									Janus.debug("We got data from the DataChannel! " + data);
+									Janus.debug("We got data from the DataChannel!", data);
 									$('#datarecv').val(data);
 								},
 								oncleanup: function() {
 									Janus.log(" ::: Got a cleanup notification :::");
-									if(spinner !== null && spinner !== undefined)
+									if(spinner)
 										spinner.stop();
 									spinner = null;
 									if(bitrateTimer)
@@ -458,7 +455,7 @@ function addSimulcastButtons(temporal) {
 			if(!$('#sl-1').hasClass('btn-success'))
 				$('#sl-1').removeClass('btn-primary btn-info').addClass('btn-primary');
 			$('#sl-0').removeClass('btn-primary btn-info btn-success').addClass('btn-info');
-			echotest.send({message: { substream: 0 }});
+			echotest.send({ message: { substream: 0 }});
 		});
 	$('#sl-1').removeClass('btn-primary btn-success').addClass('btn-primary')
 		.unbind('click').click(function() {
@@ -468,7 +465,7 @@ function addSimulcastButtons(temporal) {
 			$('#sl-1').removeClass('btn-primary btn-info btn-success').addClass('btn-info');
 			if(!$('#sl-0').hasClass('btn-success'))
 				$('#sl-0').removeClass('btn-primary btn-info').addClass('btn-primary');
-			echotest.send({message: { substream: 1 }});
+			echotest.send({ message: { substream: 1 }});
 		});
 	$('#sl-2').removeClass('btn-primary btn-success').addClass('btn-primary')
 		.unbind('click').click(function() {
@@ -478,7 +475,7 @@ function addSimulcastButtons(temporal) {
 				$('#sl-1').removeClass('btn-primary btn-info').addClass('btn-primary');
 			if(!$('#sl-0').hasClass('btn-success'))
 				$('#sl-0').removeClass('btn-primary btn-info').addClass('btn-primary');
-			echotest.send({message: { substream: 2 }});
+			echotest.send({ message: { substream: 2 }});
 		});
 	if(!temporal)	// No temporal layer support
 		return;
@@ -491,7 +488,7 @@ function addSimulcastButtons(temporal) {
 			if(!$('#tl-1').hasClass('btn-success'))
 				$('#tl-1').removeClass('btn-primary btn-info').addClass('btn-primary');
 			$('#tl-0').removeClass('btn-primary btn-info btn-success').addClass('btn-info');
-			echotest.send({message: { temporal: 0 }});
+			echotest.send({ message: { temporal: 0 }});
 		});
 	$('#tl-1').removeClass('btn-primary btn-success').addClass('btn-primary')
 		.unbind('click').click(function() {
@@ -501,7 +498,7 @@ function addSimulcastButtons(temporal) {
 			$('#tl-1').removeClass('btn-primary btn-info').addClass('btn-info');
 			if(!$('#tl-0').hasClass('btn-success'))
 				$('#tl-0').removeClass('btn-primary btn-info').addClass('btn-primary');
-			echotest.send({message: { temporal: 1 }});
+			echotest.send({ message: { temporal: 1 }});
 		});
 	$('#tl-2').removeClass('btn-primary btn-success').addClass('btn-primary')
 		.unbind('click').click(function() {
@@ -511,7 +508,7 @@ function addSimulcastButtons(temporal) {
 				$('#tl-1').removeClass('btn-primary btn-info').addClass('btn-primary');
 			if(!$('#tl-0').hasClass('btn-success'))
 				$('#tl-0').removeClass('btn-primary btn-info').addClass('btn-primary');
-			echotest.send({message: { temporal: 2 }});
+			echotest.send({ message: { temporal: 2 }});
 		});
 }
 
