@@ -125,6 +125,9 @@ $(document).ready(function() {
 										$.unblockUI();
 									}
 								},
+								iceState: function(state) {
+									Janus.log("ICE state changed to " + state);
+								},
 								mediaState: function(medium, on) {
 									Janus.log("Janus " + (on ? "started" : "stopped") + " receiving our " + medium);
 								},
@@ -145,16 +148,15 @@ $(document).ready(function() {
 											Janus.log("Capping bandwidth to " + bitrate + " via REMB");
 										}
 										$('#bitrateset').html($(this).html() + '<span class="caret"></span>').parent().removeClass('open');
-										sfutest.send({"message": { "request": "configure", "bitrate": bitrate }});
+										sfutest.send({ message: { request: "configure", bitrate: bitrate }});
 										return false;
 									});
 								},
 								onmessage: function(msg, jsep) {
-									Janus.debug(" ::: Got a message (publisher) :::");
-									Janus.debug(msg);
+									Janus.debug(" ::: Got a message (publisher) :::", msg);
 									var event = msg["videoroom"];
 									Janus.debug("Event: " + event);
-									if(event != undefined && event != null) {
+									if(event) {
 										if(event === "joined") {
 											// Publisher/manager created, negotiate WebRTC and attach to existing feeds, if any
 											myid = msg["id"];
@@ -162,10 +164,9 @@ $(document).ready(function() {
 											Janus.log("Successfully joined room " + msg["room"] + " with ID " + myid);
 											publishOwnFeed(true);
 											// Any new feed to attach to?
-											if(msg["publishers"] !== undefined && msg["publishers"] !== null) {
+											if(msg["publishers"]) {
 												var list = msg["publishers"];
-												Janus.debug("Got a list of available publishers/feeds:");
-												Janus.debug(list);
+												Janus.debug("Got a list of available publishers/feeds:", list);
 												for(var f in list) {
 													var id = list[f]["id"];
 													var display = list[f]["display"];
@@ -183,10 +184,9 @@ $(document).ready(function() {
 											});
 										} else if(event === "event") {
 											// Any new feed to attach to?
-											if(msg["publishers"] !== undefined && msg["publishers"] !== null) {
+											if(msg["publishers"]) {
 												var list = msg["publishers"];
-												Janus.debug("Got a list of available publishers/feeds:");
-												Janus.debug(list);
+												Janus.debug("Got a list of available publishers/feeds:", list);
 												for(var f in list) {
 													var id = list[f]["id"];
 													var display = list[f]["display"];
@@ -195,13 +195,13 @@ $(document).ready(function() {
 													Janus.debug("  >> [" + id + "] " + display + " (audio: " + audio + ", video: " + video + ")");
 													newRemoteFeed(id, display, audio, video);
 												}
-											} else if(msg["leaving"] !== undefined && msg["leaving"] !== null) {
+											} else if(msg["leaving"]) {
 												// One of the publishers has gone away?
 												var leaving = msg["leaving"];
 												Janus.log("Publisher left: " + leaving);
 												var remoteFeed = null;
 												for(var i=1; i<6; i++) {
-													if(feeds[i] != null && feeds[i] != undefined && feeds[i].rfid == leaving) {
+													if(feeds[i] && feeds[i].rfid == leaving) {
 														remoteFeed = feeds[i];
 														break;
 													}
@@ -213,7 +213,7 @@ $(document).ready(function() {
 													feeds[remoteFeed.rfindex] = null;
 													remoteFeed.detach();
 												}
-											} else if(msg["unpublished"] !== undefined && msg["unpublished"] !== null) {
+											} else if(msg["unpublished"]) {
 												// One of the publishers has unpublished?
 												var unpublished = msg["unpublished"];
 												Janus.log("Publisher left: " + unpublished);
@@ -224,7 +224,7 @@ $(document).ready(function() {
 												}
 												var remoteFeed = null;
 												for(var i=1; i<6; i++) {
-													if(feeds[i] != null && feeds[i] != undefined && feeds[i].rfid == unpublished) {
+													if(feeds[i] && feeds[i].rfid == unpublished) {
 														remoteFeed = feeds[i];
 														break;
 													}
@@ -236,7 +236,7 @@ $(document).ready(function() {
 													feeds[remoteFeed.rfindex] = null;
 													remoteFeed.detach();
 												}
-											} else if(msg["error"] !== undefined && msg["error"] !== null) {
+											} else if(msg["error"]) {
 												if(msg["error_code"] === 426) {
 													// This is a "no such room" error: give a more meaningful description
 													bootbox.alert(
@@ -251,10 +251,9 @@ $(document).ready(function() {
 											}
 										}
 									}
-									if(jsep !== undefined && jsep !== null) {
-										Janus.debug("Handling SDP as well...");
-										Janus.debug(jsep);
-										sfutest.handleRemoteJsep({jsep: jsep});
+									if(jsep) {
+										Janus.debug("Handling SDP as well...", jsep);
+										sfutest.handleRemoteJsep({ jsep: jsep });
 										// Check if any of the media we wanted to publish has
 										// been rejected (e.g., wrong or unsupported codec)
 										var audio = msg["audio_codec"];
@@ -277,9 +276,8 @@ $(document).ready(function() {
 									}
 								},
 								onlocalstream: function(stream) {
-									Janus.debug(" ::: Got a local stream :::");
+									Janus.debug(" ::: Got a local stream :::", stream);
 									mystream = stream;
-									Janus.debug(stream);
 									$('#videojoin').hide();
 									$('#videos').removeClass('hide').show();
 									if($('#myvideo').length === 0) {
@@ -306,7 +304,7 @@ $(document).ready(function() {
 										});
 									}
 									var videoTracks = stream.getVideoTracks();
-									if(videoTracks === null || videoTracks === undefined || videoTracks.length === 0) {
+									if(!videoTracks || videoTracks.length === 0) {
 										// No webcam
 										$('#myvideo').hide();
 										if($('#videolocal .no-video-container').length === 0) {
@@ -385,9 +383,14 @@ function registerUsername() {
 			$('#register').removeAttr('disabled').click(registerUsername);
 			return;
 		}
-		var register = { "request": "join", "room": myroom, "ptype": "publisher", "display": username };
+		var register = {
+			request: "join",
+			room: myroom,
+			ptype: "publisher",
+			display: username
+		};
 		myusername = username;
-		sfutest.send({"message": register});
+		sfutest.send({ message: register });
 	}
 }
 
@@ -404,9 +407,8 @@ function publishOwnFeed(useAudio) {
 			simulcast: doSimulcast,
 			simulcast2: doSimulcast2,
 			success: function(jsep) {
-				Janus.debug("Got publisher SDP!");
-				Janus.debug(jsep);
-				var publish = { "request": "configure", "audio": useAudio, "video": true };
+				Janus.debug("Got publisher SDP!", jsep);
+				var publish = { request: "configure", audio: useAudio, video: true };
 				// You can force a specific codec to use when publishing by using the
 				// audiocodec and videocodec properties, for instance:
 				// 		publish["audiocodec"] = "opus"
@@ -417,14 +419,14 @@ function publishOwnFeed(useAudio) {
 				// so the browser supports it), and (2) the codec is in the list of
 				// allowed codecs in a room. With respect to the point (2) above,
 				// refer to the text in janus.plugin.videoroom.jcfg for more details
-				sfutest.send({"message": publish, "jsep": jsep});
+				sfutest.send({ message: publish, jsep: jsep });
 			},
 			error: function(error) {
 				Janus.error("WebRTC error:", error);
-				if (useAudio) {
+				if(useAudio) {
 					 publishOwnFeed(false);
 				} else {
-					bootbox.alert("WebRTC error... " + JSON.stringify(error));
+					bootbox.alert("WebRTC error... " + error.message);
 					$('#publish').removeAttr('disabled').click(function() { publishOwnFeed(true); });
 				}
 			}
@@ -445,8 +447,8 @@ function toggleMute() {
 function unpublishOwnFeed() {
 	// Unpublish our stream
 	$('#unpublish').attr('disabled', true).unbind('click');
-	var unpublish = { "request": "unpublish" };
-	sfutest.send({"message": unpublish});
+	var unpublish = { request: "unpublish" };
+	sfutest.send({ message: unpublish });
 }
 
 function newRemoteFeed(id, display, audio, video) {
@@ -462,7 +464,13 @@ function newRemoteFeed(id, display, audio, video) {
 				Janus.log("Plugin attached! (" + remoteFeed.getPlugin() + ", id=" + remoteFeed.getId() + ")");
 				Janus.log("  -- This is a subscriber");
 				// We wait for the plugin to send us an offer
-				var subscribe = { "request": "join", "room": myroom, "ptype": "subscriber", "feed": id, "private_id": mypvtid };
+				var subscribe = {
+					request: "join",
+					room: myroom,
+					ptype: "subscriber",
+					feed: id,
+					private_id: mypvtid
+				};
 				// In case you don't want to receive audio, video or data, even if the
 				// publisher is sending them, set the 'offer_audio', 'offer_video' or
 				// 'offer_data' properties to false (they're true by default), e.g.:
@@ -476,24 +484,23 @@ function newRemoteFeed(id, display, audio, video) {
 					subscribe["offer_video"] = false;
 				}
 				remoteFeed.videoCodec = video;
-				remoteFeed.send({"message": subscribe});
+				remoteFeed.send({ message: subscribe });
 			},
 			error: function(error) {
 				Janus.error("  -- Error attaching plugin...", error);
 				bootbox.alert("Error attaching plugin... " + error);
 			},
 			onmessage: function(msg, jsep) {
-				Janus.debug(" ::: Got a message (subscriber) :::");
-				Janus.debug(msg);
+				Janus.debug(" ::: Got a message (subscriber) :::", msg);
 				var event = msg["videoroom"];
 				Janus.debug("Event: " + event);
-				if(msg["error"] !== undefined && msg["error"] !== null) {
+				if(msg["error"]) {
 					bootbox.alert(msg["error"]);
-				} else if(event != undefined && event != null) {
+				} else if(event) {
 					if(event === "attached") {
 						// Subscriber created and attached
 						for(var i=1;i<6;i++) {
-							if(feeds[i] === undefined || feeds[i] === null) {
+							if(!feeds[i]) {
 								feeds[i] = remoteFeed;
 								remoteFeed.rfindex = i;
 								break;
@@ -501,7 +508,7 @@ function newRemoteFeed(id, display, audio, video) {
 						}
 						remoteFeed.rfid = msg["id"];
 						remoteFeed.rfdisplay = msg["display"];
-						if(remoteFeed.spinner === undefined || remoteFeed.spinner === null) {
+						if(!remoteFeed.spinner) {
 							var target = document.getElementById('videoremote'+remoteFeed.rfindex);
 							remoteFeed.spinner = new Spinner({top:100}).spin(target);
 						} else {
@@ -526,9 +533,8 @@ function newRemoteFeed(id, display, audio, video) {
 						// What has just happened?
 					}
 				}
-				if(jsep !== undefined && jsep !== null) {
-					Janus.debug("Handling SDP as well...");
-					Janus.debug(jsep);
+				if(jsep) {
+					Janus.debug("Handling SDP as well...", jsep);
 					// Answer and attach
 					remoteFeed.createAnswer(
 						{
@@ -537,18 +543,20 @@ function newRemoteFeed(id, display, audio, video) {
 							// (obviously only works if the publisher offered them in the first place)
 							media: { audioSend: false, videoSend: false },	// We want recvonly audio/video
 							success: function(jsep) {
-								Janus.debug("Got SDP!");
-								Janus.debug(jsep);
-								var body = { "request": "start", "room": myroom };
-								remoteFeed.send({"message": body, "jsep": jsep});
+								Janus.debug("Got SDP!", jsep);
+								var body = { request: "start", room: myroom };
+								remoteFeed.send({ message: body, jsep: jsep });
 							},
 							error: function(error) {
 								Janus.error("WebRTC error:", error);
-								bootbox.alert("WebRTC error... " + JSON.stringify(error));
+								bootbox.alert("WebRTC error... " + error.message);
 							}
 						});
 				}
 			},
+			iceState: function(state) {
+				Janus.log("ICE state of this WebRTC PeerConnection (feed #" + remoteFeed.rfindex + ") changed to " + state);
+			},
 			webrtcState: function(on) {
 				Janus.log("Janus says this WebRTC PeerConnection (feed #" + remoteFeed.rfindex + ") is " + (on ? "up" : "down") + " now");
 			},
@@ -556,7 +564,7 @@ function newRemoteFeed(id, display, audio, video) {
 				// The subscriber stream is recvonly, we don't expect anything here
 			},
 			onremotestream: function(stream) {
-				Janus.debug("Remote feed #" + remoteFeed.rfindex);
+				Janus.debug("Remote feed #" + remoteFeed.rfindex + ", stream:", stream);
 				var addButtons = false;
 				if($('#remotevideo'+remoteFeed.rfindex).length === 0) {
 					addButtons = true;
@@ -568,7 +576,7 @@ function newRemoteFeed(id, display, audio, video) {
 						'<span class="label label-info hide" id="curbitrate'+remoteFeed.rfindex+'" style="position: absolute; bottom: 0px; right: 0px; margin: 15px;"></span>');
 					// Show the video, hide the spinner and show the resolution when we get a playing event
 					$("#remotevideo"+remoteFeed.rfindex).bind("playing", function () {
-						if(remoteFeed.spinner !== undefined && remoteFeed.spinner !== null)
+						if(remoteFeed.spinner)
 							remoteFeed.spinner.stop();
 						remoteFeed.spinner = null;
 						$('#waitingvideo'+remoteFeed.rfindex).remove();
@@ -589,7 +597,7 @@ function newRemoteFeed(id, display, audio, video) {
 				}
 				Janus.attachMediaStream($('#remotevideo'+remoteFeed.rfindex).get(0), stream);
 				var videoTracks = stream.getVideoTracks();
-				if(videoTracks === null || videoTracks === undefined || videoTracks.length === 0) {
+				if(!videoTracks || videoTracks.length === 0) {
 					// No remote video
 					$('#remotevideo'+remoteFeed.rfindex).hide();
 					if($('#videoremote'+remoteFeed.rfindex + ' .no-video-container').length === 0) {
@@ -622,7 +630,7 @@ function newRemoteFeed(id, display, audio, video) {
 			},
 			oncleanup: function() {
 				Janus.log(" ::: Got a cleanup notification (remote feed " + id + ") :::");
-				if(remoteFeed.spinner !== undefined && remoteFeed.spinner !== null)
+				if(remoteFeed.spinner)
 					remoteFeed.spinner.stop();
 				remoteFeed.spinner = null;
 				$('#remotevideo'+remoteFeed.rfindex).remove();
@@ -630,7 +638,7 @@ function newRemoteFeed(id, display, audio, video) {
 				$('#novideo'+remoteFeed.rfindex).remove();
 				$('#curbitrate'+remoteFeed.rfindex).remove();
 				$('#curres'+remoteFeed.rfindex).remove();
-				if(bitrateTimer[remoteFeed.rfindex] !== null && bitrateTimer[remoteFeed.rfindex] !== null)
+				if(bitrateTimer[remoteFeed.rfindex])
 					clearInterval(bitrateTimer[remoteFeed.rfindex]);
 				bitrateTimer[remoteFeed.rfindex] = null;
 				remoteFeed.simulcastStarted = false;
@@ -677,7 +685,7 @@ function addSimulcastButtons(feed, temporal) {
 			if(!$('#sl' + index + '-1').hasClass('btn-success'))
 				$('#sl' + index + '-1').removeClass('btn-primary btn-info').addClass('btn-primary');
 			$('#sl' + index + '-0').removeClass('btn-primary btn-info btn-success').addClass('btn-info');
-			feeds[index].send({message: { request: "configure", substream: 0 }});
+			feeds[index].send({ message: { request: "configure", substream: 0 }});
 		});
 	$('#sl' + index + '-1').removeClass('btn-primary btn-success').addClass('btn-primary')
 		.unbind('click').click(function() {
@@ -687,7 +695,7 @@ function addSimulcastButtons(feed, temporal) {
 			$('#sl' + index + '-1').removeClass('btn-primary btn-info btn-success').addClass('btn-info');
 			if(!$('#sl' + index + '-0').hasClass('btn-success'))
 				$('#sl' + index + '-0').removeClass('btn-primary btn-info').addClass('btn-primary');
-			feeds[index].send({message: { request: "configure", substream: 1 }});
+			feeds[index].send({ message: { request: "configure", substream: 1 }});
 		});
 	$('#sl' + index + '-2').removeClass('btn-primary btn-success').addClass('btn-primary')
 		.unbind('click').click(function() {
@@ -697,7 +705,7 @@ function addSimulcastButtons(feed, temporal) {
 				$('#sl' + index + '-1').removeClass('btn-primary btn-info').addClass('btn-primary');
 			if(!$('#sl' + index + '-0').hasClass('btn-success'))
 				$('#sl' + index + '-0').removeClass('btn-primary btn-info').addClass('btn-primary');
-			feeds[index].send({message: { request: "configure", substream: 2 }});
+			feeds[index].send({ message: { request: "configure", substream: 2 }});
 		});
 	if(!temporal)	// No temporal layer support
 		return;
@@ -710,7 +718,7 @@ function addSimulcastButtons(feed, temporal) {
 			if(!$('#tl' + index + '-1').hasClass('btn-success'))
 				$('#tl' + index + '-1').removeClass('btn-primary btn-info').addClass('btn-primary');
 			$('#tl' + index + '-0').removeClass('btn-primary btn-info btn-success').addClass('btn-info');
-			feeds[index].send({message: { request: "configure", temporal: 0 }});
+			feeds[index].send({ message: { request: "configure", temporal: 0 }});
 		});
 	$('#tl' + index + '-1').removeClass('btn-primary btn-success').addClass('btn-primary')
 		.unbind('click').click(function() {
@@ -720,7 +728,7 @@ function addSimulcastButtons(feed, temporal) {
 			$('#tl' + index + '-1').removeClass('btn-primary btn-info').addClass('btn-info');
 			if(!$('#tl' + index + '-0').hasClass('btn-success'))
 				$('#tl' + index + '-0').removeClass('btn-primary btn-info').addClass('btn-primary');
-			feeds[index].send({message: { request: "configure", temporal: 1 }});
+			feeds[index].send({ message: { request: "configure", temporal: 1 }});
 		});
 	$('#tl' + index + '-2').removeClass('btn-primary btn-success').addClass('btn-primary')
 		.unbind('click').click(function() {
@@ -730,7 +738,7 @@ function addSimulcastButtons(feed, temporal) {
 				$('#tl' + index + '-1').removeClass('btn-primary btn-info').addClass('btn-primary');
 			if(!$('#tl' + index + '-0').hasClass('btn-success'))
 				$('#tl' + index + '-0').removeClass('btn-primary btn-info').addClass('btn-primary');
-			feeds[index].send({message: { request: "configure", temporal: 2 }});
+			feeds[index].send({ message: { request: "configure", temporal: 2 }});
 		});
 }
 
