@@ -59,7 +59,8 @@
  * with \c incomingTextData() or \c incomingBinaryData
  * though, the performance impact of directly processing and manipulating
  * RTP an RTCP packets is probably too high, and so their usage is currently
- * discouraged. As an additional note, Lua scripts can also decide to
+ * discouraged. The \c dataReady() callback can be used to figure out when
+ * data can be sent. As an additional note, Lua scripts can also decide to
  * implement the functions that return information about the plugin itself,
  * namely \c getVersion() \c getVersionString() \c getDescription()
  * \c getName() \c getAuthor() and \c getPackage(). If not implemented,
@@ -218,6 +219,7 @@ void janus_lua_setup_media(janus_plugin_session *handle);
 void janus_lua_incoming_rtp(janus_plugin_session *handle, janus_plugin_rtp *packet);
 void janus_lua_incoming_rtcp(janus_plugin_session *handle, janus_plugin_rtcp *packet);
 void janus_lua_incoming_data(janus_plugin_session *handle, janus_plugin_data *packet);
+void janus_lua_data_ready(janus_plugin_session *handle);
 void janus_lua_slow_link(janus_plugin_session *handle, int uplink, int video);
 void janus_lua_hangup_media(janus_plugin_session *handle);
 void janus_lua_destroy_session(janus_plugin_session *handle, int *error);
@@ -244,6 +246,7 @@ static janus_plugin janus_lua_plugin =
 		.incoming_rtp = janus_lua_incoming_rtp,
 		.incoming_rtcp = janus_lua_incoming_rtcp,
 		.incoming_data = janus_lua_incoming_data,
+		.data_ready = janus_lua_data_ready,
 		.slow_link = janus_lua_slow_link,
 		.hangup_media = janus_lua_hangup_media,
 		.destroy_session = janus_lua_destroy_session,
@@ -289,6 +292,7 @@ static gboolean has_incoming_rtcp = FALSE;
 static gboolean has_incoming_data_legacy = FALSE,	/* Legacy callback */
 	has_incoming_text_data = FALSE,
 	has_incoming_binary_data = FALSE;
+static gboolean has_data_ready = FALSE;
 static gboolean has_slow_link = FALSE;
 /* Lua C scheduler (for coroutines) */
 static GThread *scheduler_thread = NULL;
@@ -496,11 +500,14 @@ static int janus_lua_method_pushevent(lua_State *s) {
 		asev->transaction = transaction ? g_strdup(transaction) : NULL;
 		asev->event = event;
 		asev->jsep = jsep;
+		if(json_is_true(json_object_get(jsep, "e2ee")))
+			session->e2ee = TRUE;
 		GError *error = NULL;
 		g_thread_try_new("lua pushevent", janus_lua_async_event_helper, asev, &error);
 		if(error != NULL) {
 			JANUS_LOG(LOG_ERR, "Got error %d (%s) trying to launch the Lua pushevent thread...\n",
 				error->code, error->message ? error->message : "??");
+			g_error_free(error);
 			json_decref(event);
 			json_decref(jsep);
 			g_free(asev->transaction);
@@ -944,8 +951,20 @@ static int janus_lua_method_relaytextdata(lua_State *s) {
 	}
 	janus_refcount_increase(&session->ref);
 	janus_mutex_unlock(&lua_sessions_mutex);
+	if(!g_atomic_int_get(&session->dataready)) {
+		janus_refcount_decrease(&session->ref);
+		JANUS_LOG(LOG_WARN, "Datachannel not ready yet for session %"SCNu32", dropping data\n", id);
+		lua_pushnumber(s, -1);
+		return 1;
+	}
 	/* Send the data */
-	janus_plugin_data data = { .label = NULL, .binary = FALSE, .buffer = (char *)payload, .length = len };
+	janus_plugin_data data = {
+		.label = NULL,
+		.protocol = NULL,
+		.binary = FALSE,
+		.buffer = (char *)payload,
+		.length = len
+	};
 	janus_core->relay_data(session->handle, &data);
 	janus_refcount_decrease(&session->ref);
 	lua_pushnumber(s, 0);
@@ -979,8 +998,20 @@ static int janus_lua_method_relaybinarydata(lua_State *s) {
 	}
 	janus_refcount_increase(&session->ref);
 	janus_mutex_unlock(&lua_sessions_mutex);
+	if(!g_atomic_int_get(&session->dataready)) {
+		janus_refcount_decrease(&session->ref);
+		JANUS_LOG(LOG_WARN, "Datachannel not ready yet for session %"SCNu32", dropping data\n", id);
+		lua_pushnumber(s, -1);
+		return 1;
+	}
 	/* Send the data */
-	janus_plugin_data data = { .label = NULL, .binary = TRUE, .buffer = (char *)payload, .length = len };
+	janus_plugin_data data = {
+		.label = NULL,
+		.protocol = NULL,
+		.binary = TRUE,
+		.buffer = (char *)payload,
+		.length = len
+	};
 	janus_core->relay_data(session->handle, &data);
 	janus_refcount_decrease(&session->ref);
 	lua_pushnumber(s, 0);
@@ -1013,6 +1044,7 @@ static int janus_lua_method_startrecording(lua_State *s) {
 	janus_mutex_lock(&session->rec_mutex);
 	janus_mutex_unlock(&lua_sessions_mutex);
 	/* Iterate on all arguments, to see what we're being asked to record */
+	int recordings = 0;
 	n--;
 	int i = 1;
 	janus_recorder *arc = NULL, *vrc = NULL, *drc = NULL;
@@ -1025,31 +1057,58 @@ static int janus_lua_method_startrecording(lua_State *s) {
 		const char *folder = lua_tostring(s, i);
 		i++; n--;
 		const char *filename = lua_tostring(s, i);
-		janus_recorder *rc = janus_recorder_create(folder, codec, filename);
+		if(type == NULL || codec == NULL) {
+			/* No type or codec provided, skip this */
+			continue;
+		}
+		/* Check if the codec contains some fmtp stuff too */
+		const char *c = codec, *f = NULL;
+		gchar **parts = NULL;
+		if(strstr(codec, "/fmtp=") != NULL) {
+			parts = g_strsplit(codec, "/fmtp=", 2);
+			c = parts[0];
+			f = parts[1];
+		}
+		/* Create the recorder */
+		janus_recorder *rc = janus_recorder_create_full(folder, c, f, filename);
+		if(parts != NULL)
+			g_strfreev(parts);
 		if(rc == NULL) {
 			JANUS_LOG(LOG_ERR, "Error creating '%s' recorder...\n", type);
 			goto error;
 		}
 		if(!strcasecmp(type, "audio")) {
 			if(arc != NULL || session->arc != NULL) {
-				JANUS_LOG(LOG_ERR, "Duplicate audio recording\n");
-				goto error;
+				janus_recorder_destroy(rc);
+				JANUS_LOG(LOG_WARN, "Duplicate audio recording, skipping\n");
+				continue;
 			}
+			/* If media is encrypted, mark it in the recording */
+			if(session->e2ee)
+				janus_recorder_encrypted(rc);
 			arc = rc;
 		} else if(!strcasecmp(type, "video")) {
 			if(vrc != NULL || session->vrc != NULL) {
-				JANUS_LOG(LOG_ERR, "Duplicate video recording\n");
-				goto error;
+				janus_recorder_destroy(rc);
+				JANUS_LOG(LOG_WARN, "Duplicate video recording, skipping\n");
+				continue;
 			}
+			/* If media is encrypted, mark it in the recording */
+			if(session->e2ee)
+				janus_recorder_encrypted(rc);
 			vrc = rc;
 		} else if(!strcasecmp(type, "data")) {
 			if(drc != NULL || session->drc != NULL) {
-				JANUS_LOG(LOG_ERR, "Duplicate data recording\n");
-				goto error;
+				janus_recorder_destroy(rc);
+				JANUS_LOG(LOG_WARN, "Duplicate data recording\n");
+				continue;
 			}
 			drc = rc;
 		}
+		recordings++;
 	}
+	if(recordings == 0)
+		goto error;
 	if(arc) {
 		session->arc = arc;
 	}
@@ -1293,6 +1352,9 @@ int janus_lua_init(janus_callbacks *callback, const char *config_path) {
 	lua_getglobal(lua_state, "incomingBinaryData");
 	if(lua_isfunction(lua_state, lua_gettop(lua_state)) != 0)
 		has_incoming_binary_data = TRUE;
+	lua_getglobal(lua_state, "dataReady");
+	if(lua_isfunction(lua_state, lua_gettop(lua_state)) != 0)
+		has_data_ready = TRUE;
 	lua_getglobal(lua_state, "slowLink");
 	if(lua_isfunction(lua_state, lua_gettop(lua_state)) != 0)
 		has_slow_link = TRUE;
@@ -1310,6 +1372,7 @@ int janus_lua_init(janus_callbacks *callback, const char *config_path) {
 		g_atomic_int_set(&lua_initialized, 0);
 		JANUS_LOG(LOG_ERR, "Got error %d (%s) trying to launch the Lua scheduler thread...\n",
 			error->code, error->message ? error->message : "??");
+		g_error_free(error);
 		lua_close(lua_state);
 		g_free(lua_folder);
 		g_free(lua_file);
@@ -1324,6 +1387,7 @@ int janus_lua_init(janus_callbacks *callback, const char *config_path) {
 		g_atomic_int_set(&lua_initialized, 0);
 		JANUS_LOG(LOG_ERR, "Got error %d (%s) trying to launch the Lua timer loop thread...\n",
 			error->code, error->message ? error->message : "??");
+		g_error_free(error);
 		if(timer_loop != NULL)
 			g_main_loop_unref(timer_loop);
 		if(timer_context != NULL)
@@ -1702,7 +1766,11 @@ struct janus_plugin_result *janus_lua_handle_message(janus_plugin_session *handl
 		return janus_plugin_result_new(JANUS_PLUGIN_ERROR, "No session associated with this handle", NULL);
 	}
 	char *jsep_text = jsep ? json_dumps(jsep, JSON_INDENT(0) | JSON_PRESERVE_ORDER) : NULL;
-	json_decref(jsep);
+	if(jsep != NULL) {
+		if(json_is_true(json_object_get(jsep, "e2ee")))
+			session->e2ee = TRUE;
+		json_decref(jsep);
+	}
 	/* Invoke the script function */
 	janus_mutex_lock(&lua_mutex);
 	lua_State *t = lua_newthread(lua_state);
@@ -1967,6 +2035,33 @@ void janus_lua_incoming_data(janus_plugin_session *handle, janus_plugin_data *pa
 	janus_mutex_unlock_nodebug(&session->recipients_mutex);
 }
 
+void janus_lua_data_ready(janus_plugin_session *handle) {
+	if(handle == NULL || handle->stopped || g_atomic_int_get(&lua_stopping) || !g_atomic_int_get(&lua_initialized))
+		return;
+	janus_lua_session *session = (janus_lua_session *)handle->plugin_handle;
+	if(!session) {
+		JANUS_LOG(LOG_ERR, "No session associated with this handle...\n");
+		return;
+	}
+	if(g_atomic_int_get(&session->destroyed) || g_atomic_int_get(&session->hangingup))
+		return;
+	if(g_atomic_int_compare_and_exchange(&session->dataready, 0, 1)) {
+		JANUS_LOG(LOG_INFO, "[%s-%p] Data channel available\n", JANUS_LUA_PACKAGE, handle);
+	}
+	/* Check if the Lua script wants to receive this event */
+	if(has_data_ready) {
+		/* Yep, pass the event to the Lua script and return */
+		janus_mutex_lock(&lua_mutex);
+		lua_State *t = lua_newthread(lua_state);
+		lua_getglobal(t, "dataReady");
+		lua_pushnumber(t, session->id);
+		lua_call(t, 1, 0);
+		lua_pop(lua_state, 1);
+		janus_mutex_unlock(&lua_mutex);
+		return;
+	}
+}
+
 void janus_lua_slow_link(janus_plugin_session *handle, int uplink, int video) {
 	if(handle == NULL || handle->stopped || g_atomic_int_get(&lua_stopping) || !g_atomic_int_get(&lua_initialized))
 		return;
@@ -2014,11 +2109,12 @@ void janus_lua_hangup_media(janus_plugin_session *handle) {
 		janus_refcount_decrease(&session->ref);
 		return;
 	}
-	if(g_atomic_int_add(&session->hangingup, 1)) {
+	if(!g_atomic_int_compare_and_exchange(&session->hangingup, 0, 1)) {
 		janus_refcount_decrease(&session->ref);
 		return;
 	}
 	g_atomic_int_set(&session->started, 0);
+	g_atomic_int_set(&session->dataready, 0);
 
 	/* Reset the media properties */
 	session->accept_audio = FALSE;
@@ -2030,6 +2126,7 @@ void janus_lua_hangup_media(janus_plugin_session *handle) {
 	session->bitrate = 0;
 	session->pli_freq = 0;
 	session->pli_latest = 0;
+	session->e2ee = FALSE;
 	janus_rtp_switching_context_reset(&session->rtpctx);
 
 	/* Get rid of the recipients */
@@ -2093,14 +2190,20 @@ static void janus_lua_relay_data_packet(gpointer data, gpointer user_data) {
 		return;
 	}
 	janus_lua_session *session = (janus_lua_session *)data;
-	if(!session || !session->handle || !g_atomic_int_get(&session->started) || !session->accept_data) {
+	if(!session || !session->handle || !g_atomic_int_get(&session->started) ||
+			!session->accept_data || !g_atomic_int_get(&session->dataready)) {
 		return;
 	}
 	if(janus_core != NULL) {
 		JANUS_LOG(LOG_VERB, "Forwarding %s DataChannel message (%d bytes) to session %"SCNu32"\n",
 			packet->textdata ? "text" : "binary", packet->length, session->id);
-		janus_plugin_data data = { .label = NULL, .binary = !packet->textdata,
-			.buffer = (char *)packet->data, .length = packet->length };
+		janus_plugin_data data = {
+			.label = NULL,
+			.protocol = NULL,
+			.binary = !packet->textdata,
+			.buffer = (char *)packet->data,
+			.length = packet->length
+		};
 		janus_core->relay_data(session->handle, &data);
 	}
 	return;
