@@ -181,7 +181,8 @@
 	"record" : true|false,
 	"filename" : <base path/filename to use for the recording>,
 	"substream" : <substream to receive (0-2), in case simulcasting is enabled>,
-	"temporal" : <temporal layers to receive (0-2), in case simulcasting is enabled>
+	"temporal" : <temporal layers to receive (0-2), in case simulcasting is enabled>,
+	"fallback" : <How much time (in us, default 250000) without receiving packets will make us drop to the substream below>
 }
 \endverbatim
  *
@@ -291,6 +292,7 @@ void janus_videocall_setup_media(janus_plugin_session *handle);
 void janus_videocall_incoming_rtp(janus_plugin_session *handle, janus_plugin_rtp *packet);
 void janus_videocall_incoming_rtcp(janus_plugin_session *handle, janus_plugin_rtcp *packet);
 void janus_videocall_incoming_data(janus_plugin_session *handle, janus_plugin_data *packet);
+void janus_videocall_data_ready(janus_plugin_session *handle);
 void janus_videocall_slow_link(janus_plugin_session *handle, int uplink, int video);
 void janus_videocall_hangup_media(janus_plugin_session *handle);
 void janus_videocall_destroy_session(janus_plugin_session *handle, int *error);
@@ -316,6 +318,7 @@ static janus_plugin janus_videocall_plugin =
 		.incoming_rtp = janus_videocall_incoming_rtp,
 		.incoming_rtcp = janus_videocall_incoming_rtcp,
 		.incoming_data = janus_videocall_incoming_data,
+		.data_ready = janus_videocall_data_ready,
 		.slow_link = janus_videocall_slow_link,
 		.hangup_media = janus_videocall_hangup_media,
 		.destroy_session = janus_videocall_destroy_session,
@@ -382,10 +385,13 @@ typedef struct janus_videocall_session {
 	janus_recorder *arc;	/* The Janus recorder instance for this user's audio, if enabled */
 	janus_recorder *vrc;	/* The Janus recorder instance for this user's video, if enabled */
 	janus_recorder *drc;	/* The Janus recorder instance for this user's data, if enabled */
+	gboolean e2ee;			/* Whether media is encrypted, e.g., using Insertable Streams */
 	janus_mutex rec_mutex;	/* Mutex to protect the recorders from race conditions */
 	volatile gint incall;
+	volatile gint dataready;
 	volatile gint hangingup;
 	volatile gint destroyed;
+	janus_mutex mutex;
 	janus_refcount ref;
 } janus_videocall_session;
 static GHashTable *sessions = NULL, *usernames = NULL;
@@ -501,6 +507,7 @@ int janus_videocall_init(janus_callbacks *callback, const char *config_path) {
 		g_atomic_int_set(&initialized, 0);
 		JANUS_LOG(LOG_ERR, "Got error %d (%s) trying to launch the VideoCall handler thread...\n",
 			error->code, error->message ? error->message : "??");
+		g_error_free(error);
 		return -1;
 	}
 	JANUS_LOG(LOG_INFO, "%s initialized!\n", JANUS_VIDEOCALL_NAME);
@@ -587,6 +594,7 @@ void janus_videocall_create_session(janus_plugin_session *handle, int *error) {
 	janus_rtp_switching_context_reset(&session->context);
 	janus_rtp_simulcasting_context_reset(&session->sim_context);
 	janus_vp8_simulcast_context_reset(&session->vp8_context);
+	janus_mutex_init(&session->mutex);
 	janus_mutex_init(&session->rec_mutex);
 	g_atomic_int_set(&session->incall, 0);
 	g_atomic_int_set(&session->hangingup, 0);
@@ -665,6 +673,8 @@ json_t *janus_videocall_query_session(janus_plugin_session *handle) {
 		json_object_set_new(info, "substream-target", json_integer(session->sim_context.substream_target));
 		json_object_set_new(info, "temporal-layer", json_integer(session->sim_context.templayer));
 		json_object_set_new(info, "temporal-layer-target", json_integer(session->sim_context.templayer_target));
+		if(session->sim_context.drop_trigger > 0)
+			json_object_set_new(info, "fallback", json_integer(session->sim_context.drop_trigger));
 	}
 	if(session->arc || session->vrc || session->drc) {
 		json_t *recording = json_object();
@@ -677,6 +687,8 @@ json_t *janus_videocall_query_session(janus_plugin_session *handle) {
 		json_object_set_new(info, "recording", recording);
 	}
 	json_object_set_new(info, "incall", json_integer(g_atomic_int_get(&session->incall)));
+	if(session->e2ee)
+		json_object_set_new(info, "e2ee", json_true());
 	json_object_set_new(info, "hangingup", json_integer(g_atomic_int_get(&session->hangingup)));
 	json_object_set_new(info, "destroyed", json_integer(g_atomic_int_get(&session->destroyed)));
 	janus_refcount_decrease(&session->ref);
@@ -860,7 +872,8 @@ void janus_videocall_incoming_data(janus_plugin_session *handle, janus_plugin_da
 			JANUS_LOG(LOG_ERR, "Session has no peer...\n");
 			return;
 		}
-		if(g_atomic_int_get(&session->destroyed) || g_atomic_int_get(&peer->destroyed))
+		if(g_atomic_int_get(&session->destroyed) || g_atomic_int_get(&peer->destroyed) ||
+				!g_atomic_int_get(&peer->dataready))
 			return;
 		if(packet->buffer == NULL || packet->length == 0)
 			return;
@@ -874,6 +887,7 @@ void janus_videocall_incoming_data(janus_plugin_session *handle, janus_plugin_da
 		/* Forward the packet to the peer */
 		janus_plugin_data r = {
 			.label = label,
+			.protocol = NULL,
 			.binary = packet->binary,
 			.buffer = buf,
 			.length = len
@@ -882,6 +896,19 @@ void janus_videocall_incoming_data(janus_plugin_session *handle, janus_plugin_da
 	}
 }
 
+void janus_videocall_data_ready(janus_plugin_session *handle) {
+	if(handle == NULL || g_atomic_int_get(&handle->stopped) ||
+			g_atomic_int_get(&stopping) || !g_atomic_int_get(&initialized) || !gateway)
+		return;
+	/* Data channels are writable */
+	janus_videocall_session *session = (janus_videocall_session *)handle->plugin_handle;
+	if(!session || g_atomic_int_get(&session->destroyed) || g_atomic_int_get(&session->hangingup))
+		return;
+	if(g_atomic_int_compare_and_exchange(&session->dataready, 0, 1)) {
+		JANUS_LOG(LOG_INFO, "[%s-%p] Data channel available\n", JANUS_VIDEOCALL_PACKAGE, handle);
+	}
+}
+
 void janus_videocall_slow_link(janus_plugin_session *handle, int uplink, int video) {
 	/* The core is informing us that our peer got or sent too many NACKs, are we pushing media too hard? */
 	if(handle == NULL || g_atomic_int_get(&handle->stopped) || g_atomic_int_get(&stopping) || !g_atomic_int_get(&initialized))
@@ -964,10 +991,12 @@ static void janus_videocall_hangup_media_internal(janus_plugin_session *handle)
 		return;
 	if(!g_atomic_int_compare_and_exchange(&session->hangingup, 0, 1))
 		return;
+	g_atomic_int_set(&session->dataready, 0);
 	/* Get rid of the recorders, if available */
 	janus_mutex_lock(&session->rec_mutex);
 	janus_videocall_recorder_close(session);
 	janus_mutex_unlock(&session->rec_mutex);
+	janus_mutex_lock(&session->mutex);
 	janus_videocall_session *peer = session->peer;
 	session->peer = NULL;
 	if(peer) {
@@ -990,7 +1019,9 @@ static void janus_videocall_hangup_media_internal(janus_plugin_session *handle)
 			json_object_set_new(info, "reason", json_string("Remote WebRTC hangup"));
 			gateway->notify_event(&janus_videocall_plugin, peer->handle, info);
 		}
+		janus_refcount_decrease(&peer->ref);
 	}
+	janus_mutex_unlock(&session->mutex);
 	/* Reset controls */
 	session->has_audio = FALSE;
 	session->has_video = FALSE;
@@ -1001,6 +1032,7 @@ static void janus_videocall_hangup_media_internal(janus_plugin_session *handle)
 	session->vcodec = JANUS_VIDEOCODEC_NONE;
 	session->bitrate = 0;
 	session->peer_bitrate = 0;
+	session->e2ee = FALSE;
 	int i=0;
 	for(i=0; i<3; i++) {
 		session->ssrc[i] = 0;
@@ -1068,6 +1100,9 @@ static void *janus_videocall_handler(void *data) {
 			goto error;
 		const char *msg_sdp_type = json_string_value(json_object_get(msg->jsep, "type"));
 		const char *msg_sdp = json_string_value(json_object_get(msg->jsep, "sdp"));
+		json_t *msg_e2ee = json_object_get(msg->jsep, "e2ee");
+		if(json_is_true(msg_e2ee))
+			session->e2ee = TRUE;
 		json_t *request = json_object_get(root, "request");
 		const char *request_text = json_string_value(request);
 		json_t *result = NULL;
@@ -1240,8 +1275,14 @@ static void *janus_videocall_handler(void *data) {
 				}
 				janus_sdp_destroy(offer);
 				g_atomic_int_set(&peer->incall, 1);
+				janus_refcount_increase(&session->ref);
+				janus_refcount_increase(&peer->ref);
+				janus_mutex_lock(&session->mutex);
 				session->peer = peer;
+				janus_mutex_unlock(&session->mutex);
+				janus_mutex_lock(&peer->mutex);
 				peer->peer = session;
+				janus_mutex_unlock(&peer->mutex);
 				session->has_audio = (strstr(msg_sdp, "m=audio") != NULL);
 				session->has_video = (strstr(msg_sdp, "m=video") != NULL);
 				session->has_data = (strstr(msg_sdp, "DTLS/SCTP") != NULL);
@@ -1265,6 +1306,8 @@ static void *janus_videocall_handler(void *data) {
 				json_object_set_new(calling, "username", json_string(session->username));
 				json_object_set_new(call, "result", calling);
 				json_t *jsep = json_pack("{ssss}", "type", msg_sdp_type, "sdp", msg_sdp);
+				if(session->e2ee)
+					json_object_set_new(jsep, "e2ee", json_true());
 				g_atomic_int_set(&session->hangingup, 0);
 				int ret = gateway->push_event(peer->handle, &janus_videocall_plugin, NULL, call, jsep);
 				JANUS_LOG(LOG_VERB, "  >> Pushing event to peer: %d (%s)\n", ret, janus_get_api_error(ret));
@@ -1354,6 +1397,8 @@ static void *janus_videocall_handler(void *data) {
 			janus_sdp_destroy(answer);
 			/* Send SDP to our peer */
 			json_t *jsep = json_pack("{ssss}", "type", msg_sdp_type, "sdp", msg_sdp);
+			if(session->e2ee)
+				json_object_set_new(jsep, "e2ee", json_true());
 			json_t *call = json_object();
 			json_object_set_new(call, "videocall", json_string("event"));
 			json_t *calling = json_object();
@@ -1412,6 +1457,13 @@ static void *janus_videocall_handler(void *data) {
 				g_snprintf(error_cause, 512, "Invalid value (temporal should be 0, 1 or 2)");
 				goto error;
 			}
+			json_t *fallback = json_object_get(root, "fallback");
+			if(fallback && (!json_is_integer(fallback) || json_integer_value(fallback) < 0)) {
+				JANUS_LOG(LOG_ERR, "Invalid element (fallback should be a positive integer)\n");
+				error_code = JANUS_VIDEOCALL_ERROR_INVALID_ELEMENT;
+				g_snprintf(error_cause, 512, "Invalid value (fallback should be a positive integer)");
+				goto error;
+			}
 			if(audio) {
 				session->audio_active = json_is_true(audio);
 				JANUS_LOG(LOG_VERB, "Setting audio property: %s\n", session->audio_active ? "true" : "false");
@@ -1431,6 +1483,12 @@ static void *janus_videocall_handler(void *data) {
 				gateway->send_remb(session->handle, session->bitrate ? session->bitrate : 10000000);
 			}
 			janus_videocall_session *peer = session->peer;
+			if(fallback) {
+				JANUS_LOG(LOG_VERB, "Setting fallback timer (simulcast): %lld (was %"SCNu32")\n",
+					json_integer_value(fallback) ? json_integer_value(fallback) : 250000,
+					session->sim_context.drop_trigger ? session->sim_context.drop_trigger : 250000);
+				session->sim_context.drop_trigger = json_integer_value(fallback);
+			}
 			if(substream) {
 				session->sim_context.substream_target = json_integer_value(substream);
 				JANUS_LOG(LOG_VERB, "Setting video SSRC to let through (simulcast): %"SCNu32" (index %d, was %d)\n",
@@ -1493,7 +1551,8 @@ static void *janus_videocall_handler(void *data) {
 					char filename[255];
 					gint64 now = janus_get_real_time();
 					if(session->has_audio) {
-						/* FIXME We assume we're recording Opus, here */
+						/* Prepare an audio recording */
+						janus_recorder *rc = NULL;
 						memset(filename, 0, 255);
 						if(recording_base) {
 							/* Use the filename and path we have been provided */
@@ -1515,9 +1574,14 @@ static void *janus_videocall_handler(void *data) {
 								JANUS_LOG(LOG_ERR, "Couldn't open an audio recording file for this VideoCall user!\n");
 							}
 						}
+						/* If media is encrypted, mark it in the recording */
+						if(session->e2ee)
+							janus_recorder_encrypted(rc);
+						session->arc = rc;
 					}
 					if(session->has_video) {
-						/* FIXME We assume we're recording VP8, here */
+						/* Prepare a video recording */
+						janus_recorder *rc = NULL;
 						memset(filename, 0, 255);
 						if(recording_base) {
 							/* Use the filename and path we have been provided */
@@ -1542,14 +1606,20 @@ static void *janus_videocall_handler(void *data) {
 						/* Send a PLI */
 						JANUS_LOG(LOG_VERB, "Recording video, sending a PLI to kickstart it\n");
 						gateway->send_pli(session->handle);
+						/* If media is encrypted, mark it in the recording */
+						if(session->e2ee)
+							janus_recorder_encrypted(rc);
+						session->vrc = rc;
 					}
 					if(session->has_data) {
+						/* Prepare a data recording */
+						janus_recorder *rc = NULL;
 						memset(filename, 0, 255);
 						if(recording_base) {
 							/* Use the filename and path we have been provided */
 							g_snprintf(filename, 255, "%s-data", recording_base);
-							session->drc = janus_recorder_create(NULL, "text", filename);
-							if(session->drc == NULL) {
+							rc = janus_recorder_create(NULL, "text", filename);
+							if(rc == NULL) {
 								/* FIXME We should notify the fact the recorder could not be created */
 								JANUS_LOG(LOG_ERR, "Couldn't open a data recording file for this VideoCall user!\n");
 							}
@@ -1559,12 +1629,14 @@ static void *janus_videocall_handler(void *data) {
 								session->username ? session->username : "unknown",
 								(peer && peer->username) ? peer->username : "unknown",
 								now);
-							session->drc = janus_recorder_create(NULL, "text", filename);
-							if(session->drc == NULL) {
+							rc = janus_recorder_create(NULL, "text", filename);
+							if(rc == NULL) {
 								/* FIXME We should notify the fact the recorder could not be created */
 								JANUS_LOG(LOG_ERR, "Couldn't open a data recording file for this VideoCall user!\n");
 							}
 						}
+						/* Media encryption doesn't apply to data channels */
+						session->drc = rc;
 					}
 				}
 				janus_mutex_unlock(&session->rec_mutex);
