@@ -116,7 +116,9 @@ function handleMessage(id, tr, msg, jsep)
 			end
 			local offer = sdp.parse(cojsep.sdp)
 			logger.print("Got offer: " .. sdp.render(offer))
-			local answer = sdp.generateAnswer(offer, { audio = true, video = true, data = true })
+			local answer = sdp.generateAnswer(offer, { audio = true, video = true, data = true,
+				audioCodec = comsg["audiocodec"], videoCodec = comsg["videocodec"],
+				vp9Profile = comsg["videoprofile"], h264Profile = comsg["videoprofile"] })
 			logger.print("Generated answer: " .. sdp.render(answer))
 			logger.print("Processing request: " .. dumpTable(comsg))
 			processRequest(id, comsg)
@@ -181,6 +183,14 @@ function incomingBinaryData(id, buf, len)
 	relayBinaryData(id, buf, len);
 end
 
+function dataReady(id)
+	-- This callback is invoked when the datachannel first becomes
+	-- available (meaning you should never send data before it has been
+	-- invoked at least once), but also when the datachannel is ready to
+	-- receive more data (buffers are empty), which means it can be used
+	-- to throttle outgoing data and not send too much at a time.
+end
+
 function resumeScheduler()
 	-- This is the function responsible for resuming coroutines associated
 	-- with whatever is relevant to the Lua script, e.g., for this script,
@@ -234,6 +244,11 @@ function processRequest(id, msg)
 		if fnbase == nil then
 			fnbase = "lua-echotest-" .. id .. "-" .. require 'socket'.gettime()
 		end
+		-- For the sake of simplicity, we're assuming Opus/VP8 here; in
+		-- practice, you'll need to check what was negotiated. If you
+		-- want the codec-specific info to be saved to the .mjr file as
+		-- well, you'll need to add the '/fmtp=<info>' to the codec name,
+		-- e.g.:    "vp9/fmtp=profile-id=2"
 		startRecording(id,
 			"audio", "opus", "/tmp", fnbase .. "-audio",
 			"video", "vp8", "/tmp", fnbase .. "-video",
