@@ -1,7 +1,7 @@
 /*! \file    pp-h264.c
  * \author   Lorenzo Miniero <lorenzo@meetecho.com>
  * \copyright GNU General Public License v3
- * \brief    Post-processing to generate .mp4 files
+ * \brief    Post-processing to generate .mp4 files out of H.264 frames
  * \details  Implementation of the post-processing code (based on FFmpeg)
  * needed to generate .mp4 files out of H.264 RTP frames.
  *
@@ -62,7 +62,9 @@ int janus_pp_h264_create(char *destination, char *metadata, gboolean faststart)
 	if(destination == NULL)
 		return -1;
 	/* Setup FFmpeg */
+#if ( LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(58,9,100) )
 	av_register_all();
+#endif
 	/* Adjust logging to match the postprocessor's */
 	av_log_set_level(janus_log_level <= LOG_NONE ? AV_LOG_QUIET :
 		(janus_log_level == LOG_FATAL ? AV_LOG_FATAL :
@@ -84,7 +86,8 @@ int janus_pp_h264_create(char *destination, char *metadata, gboolean faststart)
 		JANUS_LOG(LOG_ERR, "Error guessing format\n");
 		return -1;
 	}
-	snprintf(fctx->filename, sizeof(fctx->filename), "%s", destination);
+    char filename[1024];
+	snprintf(filename, sizeof(filename), "%s", destination);
 #ifdef USE_CODECPAR
 	AVCodec *codec = avcodec_find_encoder(AV_CODEC_ID_H264);
 	if(!codec) {
@@ -137,7 +140,7 @@ int janus_pp_h264_create(char *destination, char *metadata, gboolean faststart)
 	if(faststart)
 		av_dict_set(&options, "movflags", "+faststart", 0);
 
-	int res = avio_open2(&fctx->pb, fctx->filename, AVIO_FLAG_WRITE, NULL, &options);
+	int res = avio_open2(&fctx->pb, filename, AVIO_FLAG_WRITE, NULL, &options);
 	if(res < 0) {
 		JANUS_LOG(LOG_ERR, "Error opening file for output (%d)\n", res);
 		return -1;
