@@ -70,7 +70,7 @@ int janus_pp_opus_process(FILE *file, janus_pp_frame_packet *list, int *working)
 	janus_pp_frame_packet *tmp = list;
 	long int offset = 0;
 	int bytes = 0, len = 0, steps = 0, last_seq = 0;
-	uint64_t pos = 0;
+	uint64_t pos = 0, nextPos = 0;
 	uint8_t *buffer = g_malloc0(1500);
 	while(*working && tmp != NULL) {
 		if(tmp->prev != NULL && ((tmp->ts - tmp->prev->ts)/48/20 > 1)) {
@@ -85,6 +85,12 @@ int janus_pp_opus_process(FILE *file, janus_pp_frame_packet *list, int *working)
 			int i=0;
 			for(i=0; i<silence_count; i++) {
 				pos = (tmp->prev->ts - list->ts) / 48 / 20 + i + 1;
+				if(tmp->next != NULL)
+					nextPos = (tmp->next->ts - list->ts) / 48 / 20;
+				if(pos >= nextPos) {
+					JANUS_LOG(LOG_WARN, "[SKIP] pos: %06" SCNu64 ", skipping remaining silence\n", pos);
+					break;
+				}
 				op->granulepos = 960*(pos); /* FIXME: get this from the toc byte */
 				ogg_stream_packetin(stream, op);
 				ogg_write();
