@@ -48,6 +48,7 @@ static const char *default_label = "JanusDataChannel";
 
 #define SCTP_MAX_PACKET_SIZE (1<<16)
 
+/* Events we're interested in */
 static uint16_t event_types[] = {
 	SCTP_ASSOC_CHANGE,
 	SCTP_PEER_ADDR_CHANGE,
@@ -55,21 +56,50 @@ static uint16_t event_types[] = {
 	SCTP_SHUTDOWN_EVENT,
 	SCTP_ADAPTATION_INDICATION,
 	SCTP_SEND_FAILED_EVENT,
+	SCTP_SENDER_DRY_EVENT,
 	SCTP_STREAM_RESET_EVENT,
 	SCTP_STREAM_CHANGE_EVENT
 };
 
+/* Buffered message (in case we can't send right away) */
+typedef struct janus_sctp_pending_message {
+	uint16_t id;
+	gboolean textdata;
+	char *buf;
+	size_t len;
+} janus_sctp_pending_message;
+static janus_sctp_pending_message *janus_sctp_pending_message_create(uint16_t id, gboolean textdata, char *buf, size_t len) {
+	janus_sctp_pending_message *m = g_malloc(sizeof(janus_sctp_pending_message));
+	m->id = id;
+	m->textdata = textdata;
+	if(buf != NULL && len > 0) {
+		m->buf = g_malloc(len);
+		memcpy(m->buf, buf, len);
+	} else {
+		m->buf = NULL;
+		m->len = 0;
+	}
+	return m;
+}
+static void janus_sctp_pending_message_free(janus_sctp_pending_message *m) {
+	if(m != NULL) {
+		g_free(m->buf);
+		g_free(m);
+	}
+}
+
+/* usrsctp callbacks and methods */
 int janus_sctp_data_to_dtls(void *instance, void *buffer, size_t length, uint8_t tos, uint8_t set_df);
 static int janus_sctp_incoming_data(struct socket *sock, union sctp_sockstore addr, void *data, size_t datalen, struct sctp_rcvinfo rcv, int flags, void *ulp_info);
 janus_sctp_channel *janus_sctp_find_channel_by_stream(janus_sctp_association *sctp, uint16_t stream);
 janus_sctp_channel *janus_sctp_find_free_channel(janus_sctp_association *sctp);
 uint16_t janus_sctp_find_free_stream(janus_sctp_association *sctp);
 void janus_sctp_request_more_streams(janus_sctp_association *sctp);
-int janus_sctp_send_open_request_message(struct socket *sock, uint16_t stream, char *label, uint8_t unordered, uint16_t pr_policy, uint32_t pr_value);
+int janus_sctp_send_open_request_message(struct socket *sock, uint16_t stream, char *label, char *protocol, uint8_t unordered, uint16_t pr_policy, uint32_t pr_value);
 int janus_sctp_send_open_response_message(struct socket *sock, uint16_t stream);
 int janus_sctp_send_open_ack_message(struct socket *sock, uint16_t stream);
 void janus_sctp_send_deferred_messages(janus_sctp_association *sctp);
-int janus_sctp_open_channel(janus_sctp_association *sctp, char *label, uint8_t unordered, uint16_t pr_policy, uint32_t pr_value);
+int janus_sctp_open_channel(janus_sctp_association *sctp, char *label, char *protocol, uint8_t unordered, uint16_t pr_policy, uint32_t pr_value);
 int janus_sctp_send_text_or_binary(janus_sctp_association *sctp, uint16_t id, gboolean textdata, char *text, size_t length);
 void janus_sctp_reset_outgoing_stream(janus_sctp_association *sctp, uint16_t stream);
 void janus_sctp_send_outgoing_stream_reset(janus_sctp_association *sctp);
@@ -89,6 +119,13 @@ void janus_sctp_handle_remote_error_event(struct sctp_remote_error *sre);
 void janus_sctp_handle_send_failed_event(struct sctp_send_failed_event *ssfe);
 void janus_sctp_handle_notification(janus_sctp_association *sctp, union sctp_notification *notif, size_t n);
 
+/* We need to keep a map of associations with random IDs, as usrsctp will
+ * use the pointer to our structures in the actual messages instead */
+static janus_mutex sctp_mutex;
+static GHashTable *sctp_ids = NULL;
+static void janus_sctp_association_unref(janus_sctp_association *sctp);
+
+/* SCTP management code */
 static gboolean sctp_running;
 int janus_sctp_init(void) {
 	/* Initialize the SCTP stack */
@@ -101,12 +138,25 @@ int janus_sctp_init(void) {
 		JANUS_LOG(LOG_ERR, "Error creating folder %s, expect problems...\n", debug_folder);
 	}
 #endif
+
+	/* Create a map of local IDs too, to map them to our SCTP associations */
+	janus_mutex_init(&sctp_mutex);
+	sctp_ids = g_hash_table_new_full(NULL, NULL, NULL, (GDestroyNotify)janus_sctp_association_unref);
+
 	return 0;
 }
 
 void janus_sctp_deinit(void) {
 	usrsctp_finish();
 	sctp_running = FALSE;
+	janus_mutex_lock(&sctp_mutex);
+	g_hash_table_unref(sctp_ids);
+	janus_mutex_unlock(&sctp_mutex);
+}
+
+static void janus_sctp_association_unref(janus_sctp_association *sctp) {
+	if(sctp)
+		janus_refcount_decrease(&sctp->ref);
 }
 
 static void janus_sctp_association_free(const janus_refcount *sctp_ref) {
@@ -114,6 +164,8 @@ static void janus_sctp_association_free(const janus_refcount *sctp_ref) {
 	/* This association can be destroyed, free all the resources */
 	janus_refcount_decrease(&sctp->handle->ref);
 	janus_refcount_decrease(&sctp->dtls->ref);
+	if(sctp->pending_messages != NULL)
+		g_queue_free_full(sctp->pending_messages, (GDestroyNotify)janus_sctp_pending_message_free);
 #ifdef DEBUG_SCTP
 	if(sctp->debug_dump != NULL)
 		fclose(sctp->debug_dump);
@@ -173,9 +225,25 @@ janus_sctp_association *janus_sctp_association_create(janus_dtls_srtp *dtls, jan
 	}
 	sctp->stream_buffer_counter = 0;
 
-	usrsctp_register_address((void *)sctp);
+	/* Create a unique ID to map locally: this is what we'll pass to
+	 * usrsctp_socket, which means that's what we'll get in callbacks
+	 * too: we can then use the map to retrieve the actual struct */
+	janus_mutex_lock(&sctp_mutex);
+	while(sctp->map_id == 0) {
+		sctp->map_id = janus_random_uint32();
+		if(g_hash_table_lookup(sctp_ids, GUINT_TO_POINTER(sctp->map_id)) != NULL) {
+			/* ID already taken, try another one */
+			sctp->map_id = 0;
+		}
+	}
+	janus_refcount_increase(&sctp->ref);
+	g_hash_table_insert(sctp_ids, GUINT_TO_POINTER(sctp->map_id), sctp);
+	janus_mutex_unlock(&sctp_mutex);
+
+	usrsctp_register_address(GUINT_TO_POINTER(sctp->map_id));
 	usrsctp_sysctl_set_sctp_ecn_enable(0);
-	if((sock = usrsctp_socket(AF_CONN, SOCK_STREAM, IPPROTO_SCTP, janus_sctp_incoming_data, NULL, 0, (void *)sctp)) == NULL) {
+	if((sock = usrsctp_socket(AF_CONN, SOCK_STREAM, IPPROTO_SCTP, janus_sctp_incoming_data, NULL, 0,
+			GUINT_TO_POINTER(sctp->map_id))) == NULL) {
 		JANUS_LOG(LOG_ERR, "[%"SCNu64"] Error creating usrsctp socket... (%d)\n", sctp->handle_id, errno);
 		janus_sctp_association_destroy(sctp);
 		return NULL;
@@ -242,7 +310,7 @@ janus_sctp_association *janus_sctp_association_create(janus_dtls_srtp *dtls, jan
 	memset(&sconn, 0, sizeof(struct sockaddr_conn));
 	sconn.sconn_family = AF_CONN;
 	sconn.sconn_port = htons(sctp->local_port);
-	sconn.sconn_addr = (void *)sctp;
+	sconn.sconn_addr = GUINT_TO_POINTER(sctp->map_id);
 	if(usrsctp_bind(sock, (struct sockaddr *)&sconn, sizeof(struct sockaddr_conn)) < 0) {
 		JANUS_LOG(LOG_ERR, "[%"SCNu64"] Error binding client on port %"SCNu16" (%d)\n", sctp->handle_id, sctp->local_port, errno);
 		janus_sctp_association_destroy(sctp);
@@ -261,7 +329,7 @@ janus_sctp_association *janus_sctp_association_create(janus_dtls_srtp *dtls, jan
 	memset(&rconn, 0, sizeof(struct sockaddr_conn));
 	rconn.sconn_family = AF_CONN;
 	rconn.sconn_port = htons(sctp->remote_port);
-	rconn.sconn_addr = (void *)sctp;
+	rconn.sconn_addr = GUINT_TO_POINTER(sctp->map_id);
 #ifdef HAVE_SCONN_LEN
 	rconn.sconn_len = sizeof(struct sockaddr_conn);
 #endif
@@ -279,8 +347,13 @@ void janus_sctp_association_destroy(janus_sctp_association *sctp) {
 	if(sctp == NULL || !g_atomic_int_compare_and_exchange(&sctp->destroyed, 0, 1))
 		return;
 
-	usrsctp_deregister_address(sctp);
-	if (sctp->sock != NULL) {
+	if(sctp->map_id != 0) {
+		usrsctp_deregister_address(GUINT_TO_POINTER(sctp->map_id));
+		janus_mutex_lock(&sctp_mutex);
+		g_hash_table_remove(sctp_ids, GUINT_TO_POINTER(sctp->map_id));
+		janus_mutex_unlock(&sctp_mutex);
+	}
+	if(sctp->sock != NULL) {
 		usrsctp_shutdown(sctp->sock, SHUT_RDWR);
 		usrsctp_close(sctp->sock);
 	}
@@ -302,11 +375,13 @@ void janus_sctp_data_from_dtls(janus_sctp_association *sctp, char *buf, int len)
 		}
 	}
 #endif
-	usrsctp_conninput((void *)sctp, buf, len, 0);
+	usrsctp_conninput(GUINT_TO_POINTER(sctp->map_id), buf, len, 0);
 }
 
 int janus_sctp_data_to_dtls(void *instance, void *buffer, size_t length, uint8_t tos, uint8_t set_df) {
-	janus_sctp_association *sctp = (janus_sctp_association *)instance;
+	janus_mutex_lock(&sctp_mutex);
+	janus_sctp_association *sctp = (janus_sctp_association *)g_hash_table_lookup(sctp_ids, instance);
+	janus_mutex_unlock(&sctp_mutex);
 	if(sctp == NULL || sctp->handle == NULL)
 		return -1;
 	JANUS_LOG(LOG_HUGE, "[%"SCNu64"] Data from SCTP to DTLS stack: %zu bytes\n", sctp->handle_id, length);
@@ -326,7 +401,9 @@ int janus_sctp_data_to_dtls(void *instance, void *buffer, size_t length, uint8_t
 }
 
 static int janus_sctp_incoming_data(struct socket *sock, union sctp_sockstore addr, void *data, size_t datalen, struct sctp_rcvinfo rcv, int flags, void *ulp_info) {
-	janus_sctp_association *sctp = (janus_sctp_association *)ulp_info;
+	janus_mutex_lock(&sctp_mutex);
+	janus_sctp_association *sctp = (janus_sctp_association *)g_hash_table_lookup(sctp_ids, ulp_info);
+	janus_mutex_unlock(&sctp_mutex);
 	if(sctp == NULL || sctp->dtls == NULL) {
 		free(data);
 		return 0;
@@ -342,8 +419,25 @@ static int janus_sctp_incoming_data(struct socket *sock, union sctp_sockstore ad
 	return 1;
 }
 
-void janus_sctp_send_data(janus_sctp_association *sctp, char *label, gboolean textdata, char *buf, int len) {
-	if(sctp == NULL || buf == NULL || len <= 0)
+void janus_sctp_send_data(janus_sctp_association *sctp, char *label, char *protocol, gboolean textdata, char *buf, int len) {
+	if(sctp == NULL)
+		return;
+	if(sctp->pending_messages != NULL && !g_queue_is_empty(sctp->pending_messages)) {
+		/* Messages waiting in the queue, send those first */
+		janus_sctp_pending_message *m = g_queue_peek_head(sctp->pending_messages);
+		while(m != NULL) {
+			int res = janus_sctp_send_text_or_binary(sctp, m->id, m->textdata, m->buf, m->len);
+			if(res == -2) {
+				JANUS_LOG(LOG_WARN, "[%"SCNu64"] Got EAGAIN when trying to resend pending message on channel %"SCNu16"\n",
+					sctp->handle_id, m->id);
+				break;
+			}
+			(void)g_queue_pop_head(sctp->pending_messages);
+			janus_sctp_pending_message_free(m);
+			m = g_queue_peek_head(sctp->pending_messages);
+		}
+	}
+	if(buf == NULL || len <= 0)
 		return;
 	if(label == NULL)
 		label = (char *)default_label;
@@ -363,7 +457,7 @@ void janus_sctp_send_data(janus_sctp_association *sctp, char *label, gboolean te
 	if(!found) {
 		/* There's no open channel, try opening one now */
 		JANUS_LOG(LOG_VERB, "[%"SCNu64"] Creating channel '%s'...\n", sctp->handle_id, label);
-		if(janus_sctp_open_channel(sctp, label, 0, 0, 0) < 0) {
+		if(janus_sctp_open_channel(sctp, label, protocol, 0, 0, 0) < 0) {
 			JANUS_LOG(LOG_ERR, "[%"SCNu64"] Couldn't open channel...\n", sctp->handle_id);
 			return;
 		}
@@ -380,7 +474,28 @@ void janus_sctp_send_data(janus_sctp_association *sctp, char *label, gboolean te
 		}
 	}
 	/* Send the data, whether it's text or binary */
-	janus_sctp_send_text_or_binary(sctp, i, textdata, buf, len);
+	if(sctp->pending_messages != NULL && !g_queue_is_empty(sctp->pending_messages)) {
+		/* We couldn't send all pending messages, queue the new one as well */
+		if(buf != NULL && len > 0) {
+			JANUS_LOG(LOG_WARN, "[%"SCNu64"] Couldn't send all pending messages, queueing new message\n",
+				sctp->handle_id);
+			janus_sctp_pending_message *m = janus_sctp_pending_message_create(i, textdata, buf, len);
+			if(sctp->pending_messages == NULL)
+				sctp->pending_messages = g_queue_new();
+			g_queue_push_tail(sctp->pending_messages, m);
+		}
+		return;
+	}
+	int res = janus_sctp_send_text_or_binary(sctp, i, textdata, buf, len);
+	if(res == -2) {
+		/* Delivery failed with an EAGAIN, queue and retry later */
+		JANUS_LOG(LOG_WARN, "[%"SCNu64"] Got EAGAIN when trying to send message on channel %"SCNu16", retrying later\n",
+			sctp->handle_id, i);
+		janus_sctp_pending_message *m = janus_sctp_pending_message_create(i, textdata, buf, len);
+		if(sctp->pending_messages == NULL)
+			sctp->pending_messages = g_queue_new();
+		g_queue_push_tail(sctp->pending_messages, m);
+	}
 }
 
 
@@ -447,7 +562,7 @@ void janus_sctp_request_more_streams(janus_sctp_association *sctp) {
 	streams_needed = 0;
 	for(i = 0; i < NUMBER_OF_CHANNELS; i++) {
 		if((sctp->channels[i].state == DATA_CHANNEL_CONNECTING) &&
-		    (sctp->channels[i].stream == 0)) {
+			(sctp->channels[i].stream == 0)) {
 			streams_needed++;
 		}
 	}
@@ -471,7 +586,7 @@ void janus_sctp_request_more_streams(janus_sctp_association *sctp) {
 	return;
 }
 
-int janus_sctp_send_open_request_message(struct socket *sock, uint16_t stream, char *label, uint8_t unordered, uint16_t pr_policy, uint32_t pr_value) {
+int janus_sctp_send_open_request_message(struct socket *sock, uint16_t stream, char *label, char *protocol, uint8_t unordered, uint16_t pr_policy, uint32_t pr_value) {
 	/* XXX: This should be encoded in a better way */
 	janus_datachannel_open_request *req = NULL;
 	struct sctp_sndinfo sndinfo;
@@ -479,10 +594,12 @@ int janus_sctp_send_open_request_message(struct socket *sock, uint16_t stream, c
 	/* Use the default label, if none was provided */
 	if(label == NULL)
 		label = (char *)default_label;
-	guint label_size = (strlen(label)+3) & ~3;
-	JANUS_LOG(LOG_VERB, "Opening channel with label '%s' (%zu, %u with padding)\n", label, strlen(label), label_size);
+	size_t label_size = strlen(label);
+	size_t protocol_size = protocol ? strlen(protocol) : 0;
+	JANUS_LOG(LOG_VERB, "Opening channel with label '%s' (%zu, protocol %s)\n",
+		label, label_size, (protocol ? protocol : "unknown"));
 
-	req = g_malloc0(sizeof(janus_datachannel_open_request) + label_size);
+	req = g_malloc0(sizeof(janus_datachannel_open_request) + label_size + protocol_size);
 	req->msg_type = DATA_CHANNEL_OPEN_REQUEST;
 	switch (pr_policy) {
 		case SCTP_PR_SCTP_NONE:
@@ -502,7 +619,10 @@ int janus_sctp_send_open_request_message(struct socket *sock, uint16_t stream, c
 	req->priority = htons(0); /* XXX: add support */
 	req->reliability_params = htonl((uint32_t)pr_value);
 	req->label_length = htons(label_size);
-	memcpy(&req->label, label, strlen(label));
+	req->protocol_length = htons(protocol_size);
+	memcpy(req->label, label, label_size);
+	if(protocol != NULL)
+		memcpy(req->label + label_size, protocol, protocol_size);
 
 	memset(&sndinfo, 0, sizeof(struct sctp_sndinfo));
 	sndinfo.snd_sid = stream;
@@ -510,10 +630,10 @@ int janus_sctp_send_open_request_message(struct socket *sock, uint16_t stream, c
 	sndinfo.snd_ppid = htonl(DATA_CHANNEL_PPID_CONTROL);
 
 	if(usrsctp_sendv(sock,
-	                  req, sizeof(janus_datachannel_open_request) + label_size,
-	                  NULL, 0,
-	                  &sndinfo, (socklen_t)sizeof(struct sctp_sndinfo),
-	                  SCTP_SENDV_SNDINFO, 0) < 0) {
+			req, sizeof(janus_datachannel_open_request) + label_size + protocol_size,
+			NULL, 0,
+			&sndinfo, (socklen_t)sizeof(struct sctp_sndinfo),
+			SCTP_SENDV_SNDINFO, 0) < 0) {
 		JANUS_LOG(LOG_ERR, "usrsctp_sendv error (%d)\n", errno);
 		g_free(req);
 		req = NULL;
@@ -540,10 +660,10 @@ int janus_sctp_send_open_response_message(struct socket *sock, uint16_t stream)
 	sndinfo.snd_flags = SCTP_EOR;
 	sndinfo.snd_ppid = htonl(DATA_CHANNEL_PPID_CONTROL);
 	if(usrsctp_sendv(sock,
-	                  &rsp, sizeof(janus_datachannel_open_response),
-	                  NULL, 0,
-	                  &sndinfo, (socklen_t)sizeof(struct sctp_sndinfo),
-	                  SCTP_SENDV_SNDINFO, 0) < 0) {
+			&rsp, sizeof(janus_datachannel_open_response),
+			NULL, 0,
+			&sndinfo, (socklen_t)sizeof(struct sctp_sndinfo),
+			SCTP_SENDV_SNDINFO, 0) < 0) {
 		JANUS_LOG(LOG_ERR, "usrsctp_sendv error (%d)\n", errno);
 		return 0;
 	} else {
@@ -563,10 +683,10 @@ int janus_sctp_send_open_ack_message(struct socket *sock, uint16_t stream) {
 	sndinfo.snd_flags = SCTP_EOR;
 	sndinfo.snd_ppid = htonl(DATA_CHANNEL_PPID_CONTROL);
 	if(usrsctp_sendv(sock,
-	                  &ack, sizeof(janus_datachannel_ack),
-	                  NULL, 0,
-	                  &sndinfo, (socklen_t)sizeof(struct sctp_sndinfo),
-	                  SCTP_SENDV_SNDINFO, 0) < 0) {
+			&ack, sizeof(janus_datachannel_ack),
+			NULL, 0,
+			&sndinfo, (socklen_t)sizeof(struct sctp_sndinfo),
+			SCTP_SENDV_SNDINFO, 0) < 0) {
 		JANUS_LOG(LOG_ERR, "usrsctp_sendv error (%d)\n", errno);
 		return 0;
 	} else {
@@ -582,7 +702,7 @@ void janus_sctp_send_deferred_messages(janus_sctp_association *sctp) {
 		channel = &(sctp->channels[i]);
 		if(channel->flags & DATA_CHANNEL_FLAGS_SEND_REQ) {
 			if(janus_sctp_send_open_request_message(sctp->sock, channel->stream,
-					channel->label, channel->unordered, channel->pr_policy, channel->pr_value)) {
+					channel->label, channel->protocol, channel->unordered, channel->pr_policy, channel->pr_value)) {
 				channel->flags &= ~DATA_CHANNEL_FLAGS_SEND_REQ;
 			} else {
 				if(errno != EAGAIN) {
@@ -612,7 +732,7 @@ void janus_sctp_send_deferred_messages(janus_sctp_association *sctp) {
 	return;
 }
 
-int janus_sctp_open_channel(janus_sctp_association *sctp, char *label, uint8_t unordered, uint16_t pr_policy, uint32_t pr_value) {
+int janus_sctp_open_channel(janus_sctp_association *sctp, char *label, char *protocol, uint8_t unordered, uint16_t pr_policy, uint32_t pr_value) {
 	if(sctp == NULL)
 		return -1;
 	janus_sctp_channel *channel;
@@ -640,10 +760,13 @@ int janus_sctp_open_channel(janus_sctp_association *sctp, char *label, uint8_t u
 	channel->stream = stream;
 	channel->flags = 0;
 	g_snprintf(channel->label, sizeof(channel->label), "%s", (label ? label : default_label));
+	channel->protocol[0] = '\0';
+	if(protocol != NULL)
+		g_snprintf(channel->protocol, sizeof(channel->protocol), "%s", protocol);
 	if(stream == 0) {
 		janus_sctp_request_more_streams(sctp);
 	} else {
-		if(janus_sctp_send_open_request_message(sctp->sock, stream, channel->label, unordered, pr_policy, pr_value)) {
+		if(janus_sctp_send_open_request_message(sctp->sock, stream, channel->label, channel->protocol, unordered, pr_policy, pr_value)) {
 			sctp->stream_channel[stream] = channel;
 		} else {
 			if(errno == EAGAIN) {
@@ -651,6 +774,7 @@ int janus_sctp_open_channel(janus_sctp_association *sctp, char *label, uint8_t u
 				channel->flags |= DATA_CHANNEL_FLAGS_SEND_REQ;
 			} else {
 				channel->label[0] = '\0';
+				channel->protocol[0] = '\0';
 				channel->state = DATA_CHANNEL_CLOSED;
 				channel->unordered = 0;
 				channel->pr_policy = 0;
@@ -696,7 +820,12 @@ int janus_sctp_send_text_or_binary(janus_sctp_association *sctp, uint16_t id, gb
 	if(usrsctp_sendv(sctp->sock, text, length, NULL, 0,
 			&spa, (socklen_t)sizeof(struct sctp_sendv_spa),
 			SCTP_SENDV_SPA, 0) < 0) {
-		JANUS_LOG(LOG_ERR, "[%"SCNu64"] sctp_sendv error (%d)\n", sctp->handle_id, errno);
+		int res = errno;
+		if(res == EAGAIN) {
+			/* Couldn't send the message right away, add to the queue and retry later */
+			return -2;
+		}
+		JANUS_LOG(LOG_ERR, "[%"SCNu64"] sctp_sendv error (%d)\n", sctp->handle_id, res);
 		return -1;
 	}
 	JANUS_LOG(LOG_VERB, "[%"SCNu64"] Message sent on channel %"SCNu16"\n", sctp->handle_id, id);
@@ -772,6 +901,7 @@ void janus_sctp_handle_open_request_message(janus_sctp_association *sctp, janus_
 
 	if((channel = janus_sctp_find_channel_by_stream(sctp, stream))) {
 		JANUS_LOG(LOG_ERR, "[%"SCNu64"] channel %d is in state %d instead of CLOSED.\n", sctp->handle_id, channel->id, channel->state);
+		JANUS_LOG(LOG_ERR, "%.*s\n", req->label_length, req->label);
 		/* XXX: some error handling */
 		return;
 	}
@@ -828,6 +958,7 @@ void janus_sctp_handle_open_request_message(janus_sctp_association *sctp, janus_
 			/* XXX: Signal error to the other end */
 			sctp->stream_channel[stream] = NULL;
 			channel->label[0] = '\0';
+			channel->protocol[0] = '\0';
 			channel->state = DATA_CHANNEL_CLOSED;
 			channel->unordered = 0;
 			channel->pr_policy = 0;
@@ -845,10 +976,19 @@ void janus_sctp_handle_open_request_message(janus_sctp_association *sctp, janus_
 		label[len] = '\0';
 		g_snprintf(channel->label, sizeof(channel->label), "%s", label);
 	}
-	JANUS_LOG(LOG_VERB, "[%"SCNu64"] Opened channel '%s' (id=%"SCNu16") (%d/%d/%d)\n",
-		sctp->handle_id, label ? label : "??",
+	char *protocol = NULL;
+	guint plen = ntohs(req->protocol_length);
+	if(plen > 0 && plen < length) {
+		protocol = g_malloc(plen+1);
+		memcpy(protocol, req->label+len, plen);
+		protocol[plen] = '\0';
+		g_snprintf(channel->protocol, sizeof(channel->protocol), "%s", protocol);
+	}
+	JANUS_LOG(LOG_VERB, "[%"SCNu64"] Opened channel '%s' (protocol=%s, id=%"SCNu16") (%d/%d/%d)\n",
+		sctp->handle_id, label ? label : "??", protocol ? protocol : "??",
 		channel->stream, channel->unordered, channel->pr_policy, channel->pr_value);
 	g_free(label);
+	g_free(protocol);
 }
 
 void janus_sctp_handle_open_response_message(janus_sctp_association *sctp, janus_datachannel_open_response *rsp, size_t length, uint16_t stream) {
@@ -927,11 +1067,13 @@ void janus_sctp_handle_data_message(janus_sctp_association *sctp, gboolean textd
 	} else {
 		/* XXX: Protect for non 0 terminated buffer */
 		JANUS_LOG(LOG_VERB, "[%"SCNu64"] SCTP data received of length %zu on channel with id %d.\n",
-		       sctp->handle_id, length, channel->id);
+			sctp->handle_id, length, channel->id);
 		JANUS_LOG(LOG_HUGE, "[%"SCNu64"] Incoming SCTP contents: %.*s\n",
-		       sctp->handle_id, (int)length, buffer);
+			sctp->handle_id, (int)length, buffer);
 		/* Pass this to the core */
-		janus_dtls_notify_data(sctp->dtls, channel->label, textdata, buffer, (int)length);
+		janus_dtls_notify_sctp_data(sctp->dtls, channel->label,
+			strlen(channel->protocol) ? channel->protocol : NULL,
+			textdata, buffer, (int)length);
 	}
 	return;
 }
@@ -1008,7 +1150,7 @@ void janus_sctp_handle_message(janus_sctp_association *sctp, char *buffer, size_
 			break;
 		default:
 			JANUS_LOG(LOG_VERB, "[%"SCNu64"] Message of length %zu, PPID %u on stream %u received.\n",
-				   sctp->handle_id, length, ppid, stream);
+				sctp->handle_id, length, ppid, stream);
 			break;
 	}
 }
@@ -1038,10 +1180,10 @@ void janus_sctp_handle_association_change_event(struct sctp_assoc_change *sac) {
 			break;
 	}
 	JANUS_LOG(LOG_VERB, ", streams (in/out) = (%u/%u)",
-	       sac->sac_inbound_streams, sac->sac_outbound_streams);
+		sac->sac_inbound_streams, sac->sac_outbound_streams);
 	n = sac->sac_length - sizeof(struct sctp_assoc_change);
 	if(((sac->sac_state == SCTP_COMM_UP) ||
-	     (sac->sac_state == SCTP_RESTART)) && (n > 0)) {
+			(sac->sac_state == SCTP_RESTART)) && (n > 0)) {
 		JANUS_LOG(LOG_VERB, ", supports");
 		for(i = 0; i < n; i++) {
 			switch (sac->sac_info[i]) {
@@ -1066,7 +1208,7 @@ void janus_sctp_handle_association_change_event(struct sctp_assoc_change *sac) {
 			}
 		}
 	} else if(((sac->sac_state == SCTP_COMM_LOST) ||
-	            (sac->sac_state == SCTP_CANT_STR_ASSOC)) && (n > 0)) {
+			(sac->sac_state == SCTP_CANT_STR_ASSOC)) && (n > 0)) {
 		JANUS_LOG(LOG_VERB, ", ABORT =");
 		for(i = 0; i < n; i++) {
 			JANUS_LOG(LOG_VERB, " 0x%02x", sac->sac_info[i]);
@@ -1164,7 +1306,7 @@ void janus_sctp_handle_stream_reset_event(janus_sctp_association *sctp, struct s
 	}
 	JANUS_LOG(LOG_VERB, ".\n");
 	if(!(strrst->strreset_flags & SCTP_STREAM_RESET_DENIED) &&
-	    !(strrst->strreset_flags & SCTP_STREAM_RESET_FAILED)) {
+			!(strrst->strreset_flags & SCTP_STREAM_RESET_FAILED)) {
 		for(i = 0; i < n; i++) {
 			if(strrst->strreset_flags & SCTP_STREAM_RESET_INCOMING_SSN ||
 					strrst->strreset_flags & SCTP_STREAM_RESET_OUTGOING_SSN) {
@@ -1218,8 +1360,8 @@ void janus_sctp_handle_send_failed_event(struct sctp_send_failed_event *ssfe) {
 		JANUS_LOG(LOG_VERB, "(flags = %x) ", ssfe->ssfe_flags);
 	}
 	JANUS_LOG(LOG_VERB, "message with PPID = %d, SID = %d, flags: 0x%04x due to error = 0x%08x",
-	       ntohl(ssfe->ssfe_info.snd_ppid), ssfe->ssfe_info.snd_sid,
-	       ssfe->ssfe_info.snd_flags, ssfe->ssfe_error);
+		ntohl(ssfe->ssfe_info.snd_ppid), ssfe->ssfe_info.snd_sid,
+		ssfe->ssfe_info.snd_flags, ssfe->ssfe_error);
 	n = ssfe->ssfe_length - sizeof(struct sctp_send_failed_event);
 	for(i = 0; i < n; i++) {
 		JANUS_LOG(LOG_VERB, " 0x%02x", ssfe->ssfe_data[i]);
@@ -1252,8 +1394,12 @@ void janus_sctp_handle_notification(janus_sctp_association *sctp, union sctp_not
 			break;
 		case SCTP_AUTHENTICATION_EVENT:
 			break;
-		case SCTP_SENDER_DRY_EVENT:
+		case SCTP_SENDER_DRY_EVENT: {
+			/* Internal buffers empty, notify the application they can send again */
+			if(sctp != NULL && !g_atomic_int_get(&sctp->destroyed))
+				janus_dtls_sctp_data_ready(sctp->dtls);
 			break;
+		}
 		case SCTP_NOTIFICATIONS_STOPPED_EVENT:
 			break;
 		case SCTP_SEND_FAILED_EVENT:
@@ -1268,7 +1414,7 @@ void janus_sctp_handle_notification(janus_sctp_association *sctp, union sctp_not
 		case SCTP_ASSOC_RESET_EVENT:
 			break;
 		case SCTP_STREAM_CHANGE_EVENT:
-			JANUS_LOG(LOG_VERB, "Stream change (in/out) = (%u/%u)\n",
+			JANUS_LOG(LOG_VERB, "[%"SCNu64"] Stream change (in/out) = (%u/%u)\n", sctp ? sctp->handle_id : 0,
 				notif->sn_strchange_event.strchange_instrms, notif->sn_strchange_event.strchange_outstrms);
 			break;
 		default:
