@@ -13,7 +13,7 @@
  * WebSockets interface will need to include, when needed, additional
  * pieces of information like \c session_id and \c handle_id. That is,
  * where you'd send a Janus request related to a specific session to the
- * \c /janus/<session> path, with WebSockets you'd have to send the same
+ * \c /janus/\<session> path, with WebSockets you'd have to send the same
  * request with an additional \c session_id field in the JSON payload.
  * The same applies for the handle. The JavaScript library (janus.js)
  * implements all of this on the client side automatically.
@@ -271,6 +271,10 @@ static const char *janus_websockets_reason_string(enum lws_callback_reasons reas
 	return NULL;
 }
 
+#if (LWS_LIBRARY_VERSION_MAJOR >= 4)
+static lws_retry_bo_t pingpong = { 0 };
+#endif
+
 /* Helper method to return the interface associated with a local IP address */
 static char *janus_websockets_get_interface_name(const char *ip) {
 	struct ifaddrs *addrs = NULL, *iap = NULL;
@@ -304,8 +308,8 @@ static char *janus_websockets_get_interface_name(const char *ip) {
 }
 
 /* WebSockets ACL list for both Janus and Admin API */
-GList *janus_websockets_access_list = NULL, *janus_websockets_admin_access_list = NULL;
-janus_mutex access_list_mutex;
+static GList *janus_websockets_access_list = NULL, *janus_websockets_admin_access_list = NULL;
+static janus_mutex access_list_mutex;
 static void janus_websockets_allow_address(const char *ip, gboolean admin) {
 	if(ip == NULL)
 		return;
@@ -530,11 +534,21 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 		if((pingpong_trigger && !pingpong_timeout) || (!pingpong_trigger && pingpong_timeout)) {
 			JANUS_LOG(LOG_WARN, "pingpong_trigger and pingpong_timeout not both set, ignoring...\n");
 		}
-#if (LWS_LIBRARY_VERSION_MAJOR >= 2 && LWS_LIBRARY_VERSION_MINOR >= 1) || (LWS_LIBRARY_VERSION_MAJOR >= 3)
+#if (LWS_LIBRARY_VERSION_MAJOR >= 4)
+		/* libwebsockets 4 has a different API, that works differently
+		 * https://github.com/warmcat/libwebsockets/blob/master/READMEs/README.lws_retry.md */
+		if(pingpong_trigger > 0 && pingpong_timeout > 0) {
+			pingpong.secs_since_valid_ping = pingpong_trigger;
+			pingpong.secs_since_valid_hangup = pingpong_trigger + pingpong_timeout;
+			wscinfo.retry_and_idle_policy = &pingpong;
+		}
+#else
+#if (LWS_LIBRARY_VERSION_MAJOR >= 2 && LWS_LIBRARY_VERSION_MINOR >= 1) || (LWS_LIBRARY_VERSION_MAJOR == 3)
 		if(pingpong_trigger > 0 && pingpong_timeout > 0) {
 			wscinfo.ws_ping_pong_interval = pingpong_trigger;
 			wscinfo.timeout_secs = pingpong_timeout;
 		}
+#endif
 #endif
 		/* Force single-thread server */
 		wscinfo.count_threads = 1;
@@ -625,6 +639,7 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 				char *server_pem = (char *)item->value;
 				char *server_key = (char *)item->value;
 				char *password = NULL;
+				char *ciphers = NULL;
 				item = janus_config_get(config, config_certs, janus_config_type_item, "cert_key");
 				if(item && item->value)
 					server_key = (char *)item->value;
@@ -632,6 +647,9 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 				if(item && item->value)
 					password = (char *)item->value;
 				JANUS_LOG(LOG_VERB, "Using certificates:\n\t%s\n\t%s\n", server_pem, server_key);
+				item = janus_config_get(config, config_certs, janus_config_type_item, "ciphers");
+				if(item && item->value)
+					ciphers = (char *)item->value;
 				/* Prepare secure context */
 				struct lws_context_creation_info info;
 				memset(&info, 0, sizeof info);
@@ -642,6 +660,7 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 				info.ssl_cert_filepath = server_pem;
 				info.ssl_private_key_filepath = server_key;
 				info.ssl_private_key_password = password;
+				info.ssl_cipher_list = ciphers;
 				info.gid = -1;
 				info.uid = -1;
 #if LWS_LIBRARY_VERSION_MAJOR >= 2
@@ -737,6 +756,7 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 				char *server_pem = (char *)item->value;
 				char *server_key = (char *)item->value;
 				char *password = NULL;
+				char *ciphers = NULL;
 				item = janus_config_get(config, config_certs, janus_config_type_item, "cert_key");
 				if(item && item->value)
 					server_key = (char *)item->value;
@@ -744,6 +764,9 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 				if(item && item->value)
 					password = (char *)item->value;
 				JANUS_LOG(LOG_VERB, "Using certificates:\n\t%s\n\t%s\n", server_pem, server_key);
+				item = janus_config_get(config, config_certs, janus_config_type_item, "ciphers");
+				if(item && item->value)
+					ciphers = (char *)item->value;
 				/* Prepare secure context */
 				struct lws_context_creation_info info;
 				memset(&info, 0, sizeof info);
@@ -754,6 +777,7 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 				info.ssl_cert_filepath = server_pem;
 				info.ssl_private_key_filepath = server_key;
 				info.ssl_private_key_password = password;
+				info.ssl_cipher_list = ciphers;
 				info.gid = -1;
 				info.uid = -1;
 #if LWS_LIBRARY_VERSION_MAJOR >= 2
@@ -794,9 +818,11 @@ int janus_websockets_init(janus_transport_callbacks *callback, const char *confi
 	/* Start the WebSocket service thread */
 	if(ws_janus_api_enabled || ws_admin_api_enabled) {
 		ws_thread = g_thread_try_new("ws thread", &janus_websockets_thread, wsc, &error);
-		if(!ws_thread) {
+		if(error != NULL) {
 			g_atomic_int_set(&initialized, 0);
-			JANUS_LOG(LOG_ERR, "Got error %d (%s) trying to launch the WebSockets thread...\n", error->code, error->message ? error->message : "??");
+			JANUS_LOG(LOG_ERR, "Got error %d (%s) trying to launch the WebSockets thread...\n",
+				error->code, error->message ? error->message : "??");
+			g_error_free(error);
 			return -1;
 		}
 	}
@@ -1154,6 +1180,23 @@ static int janus_websockets_common_callback(
 			}
 			if(!g_atomic_int_get(&ws_client->destroyed) && !g_atomic_int_get(&stopping)) {
 				janus_mutex_lock(&ws_client->ts->mutex);
+
+				/* Check if Websockets send pipe is choked */
+				if(lws_send_pipe_choked(wsi)) {
+					if(ws_client->buffer && ws_client->bufpending > 0 && ws_client->bufoffset > 0) {
+						JANUS_LOG(LOG_WARN, "Websockets choked with buffer: %d, trying again\n", ws_client->bufpending);
+						lws_callback_on_writable(wsi);
+					} else {
+						gint qlen = g_async_queue_length(ws_client->messages);
+						JANUS_LOG(LOG_WARN, "Websockets choked with queue: %d, trying again\n", qlen);
+						if(qlen > 0) {
+							lws_callback_on_writable(wsi);
+						}
+					}
+					janus_mutex_unlock(&ws_client->ts->mutex);
+					return 0;
+				}
+
 				/* Check if we have a pending/partial write to complete first */
 				if(ws_client->buffer && ws_client->bufpending > 0 && ws_client->bufoffset > 0
 						&& !g_atomic_int_get(&ws_client->destroyed) && !g_atomic_int_get(&stopping)) {
